@@ -9,6 +9,8 @@
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2015
 # - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
 # - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
+# - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
+# - Patrick Austin, <patrick.austin@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -23,30 +25,32 @@ from rucio.core.account import account_exists
 from rucio.core.rse import get_rse_id, get_rse_name
 
 
-def get_rse_account_usage(rse):
+def get_rse_account_usage(rse, vo='def'):
     """
     Returns the account limit and usage for all for all accounts on a RSE.
 
     :param rse:      The RSE name.
+    :param vo:       The VO to act on.
     :return:         List of dictionnaries.
     """
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
 
     return [api_update_return_dict(d) for d in account_limit_core.get_rse_account_usage(rse_id=rse_id)]
 
 
-def get_local_account_limits(account):
+def get_local_account_limits(account, vo='def'):
     """
     Lists the limitation names/values for the specified account name.
 
     REST API: http://<host>:<port>/rucio/account/<account>/limits
 
     :param account:     The account name.
+    :param vo:          The VO to act on.
 
     :returns: The account limits.
     """
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
     rse_instead_id = {}
     for elem in account_limit_core.get_local_account_limits(account=account).items():
@@ -54,7 +58,7 @@ def get_local_account_limits(account):
     return rse_instead_id
 
 
-def get_local_account_limit(account, rse):
+def get_local_account_limit(account, rse, vo='def'):
     """
     Lists the limitation names/values for the specified account name and rse name.
 
@@ -62,32 +66,37 @@ def get_local_account_limit(account, rse):
 
     :param account:     The account name.
     :param rse:         The rse name.
+    :param vo:          The VO to act on.
 
     :returns: The account limit.
     """
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
     return {rse: account_limit_core.get_local_account_limit(account=account, rse_id=rse_id)}
 
 
-def get_global_account_limits(account):
+def get_global_account_limits(account, vo='def'):
     """
     Lists the limitation names/values for the specified account name.
 
     REST API: http://<host>:<port>/rucio/account/<account>/limits
 
     :param account:     The account name.
+    :param vo:          The VO to act on.
 
     :returns: The account limits.
     """
+    if account:
+        account = InternalAccount(account, vo=vo)
+    else:
+        account = InternalAccount('*', vo=vo)
 
-    account = InternalAccount(account)
     return account_limit_core.get_global_account_limits(account=account)
 
 
-def get_global_account_limit(account, rse_expression):
+def get_global_account_limit(account, rse_expression, vo='def'):
     """
     Lists the limitation names/values for the specified account name and rse expression.
 
@@ -95,15 +104,17 @@ def get_global_account_limit(account, rse_expression):
 
     :param account:         The account name.
     :param rse_expression:  The rse expression.
+    :param vo:              The VO to act on.
 
     :returns: The account limit.
     """
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
+
     return {rse_expression: account_limit_core.get_global_account_limit(account=account, rse_expression=rse_expression)}
 
 
-def set_local_account_limit(account, rse, bytes, issuer):
+def set_local_account_limit(account, rse, bytes, issuer, vo='def'):
     """
     Set an account limit..
 
@@ -111,14 +122,15 @@ def set_local_account_limit(account, rse, bytes, issuer):
     :param rse:     The rse name.
     :param bytes:   The limit in bytes.
     :param issuer:  The issuer account_core.
+    :param vo:      The VO to act on.
     """
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
 
     kwargs = {'account': account, 'rse': rse, 'rse_id': rse_id, 'bytes': bytes}
-    if not rucio.api.permission.has_permission(issuer=issuer, action='set_local_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='set_local_account_limit', kwargs=kwargs):
         raise rucio.common.exception.AccessDenied('Account %s can not set account limits.' % (issuer))
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
     if not account_exists(account=account):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
@@ -126,7 +138,7 @@ def set_local_account_limit(account, rse, bytes, issuer):
     account_limit_core.set_local_account_limit(account=account, rse_id=rse_id, bytes=bytes)
 
 
-def set_global_account_limit(account, rse_expression, bytes, issuer):
+def set_global_account_limit(account, rse_expression, bytes, issuer, vo='def'):
     """
     Set a global account limit.
 
@@ -134,13 +146,14 @@ def set_global_account_limit(account, rse_expression, bytes, issuer):
     :param rse_expression:  The rse expression.
     :param bytes:           The limit in bytes.
     :param issuer:          The issuer account_core.
+    :param vo:              The VO to act on.
     """
 
     kwargs = {'account': account, 'rse_expression': rse_expression, 'bytes': bytes}
-    if not rucio.api.permission.has_permission(issuer=issuer, action='set_global_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='set_global_account_limit', kwargs=kwargs):
         raise rucio.common.exception.AccessDenied('Account %s can not set account limits.' % (issuer))
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
     if not account_exists(account=account):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
@@ -148,23 +161,24 @@ def set_global_account_limit(account, rse_expression, bytes, issuer):
     account_limit_core.set_global_account_limit(account=account, rse_expression=rse_expression, bytes=bytes)
 
 
-def delete_local_account_limit(account, rse, issuer):
+def delete_local_account_limit(account, rse, issuer, vo='def'):
     """
     Delete an account limit..
 
     :param account: The account name.
     :param rse:     The rse name.
     :param issuer:  The issuer account_core.
+    :param vo:      The VO to act on.
 
     :returns: True if successful; False otherwise.
     """
 
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
     kwargs = {'account': account, 'rse': rse, 'rse_id': rse_id}
-    if not rucio.api.permission.has_permission(issuer=issuer, action='delete_local_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='delete_local_account_limit', kwargs=kwargs):
         raise rucio.common.exception.AccessDenied('Account %s can not delete account limits.' % (issuer))
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
     if not account_exists(account=account):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
@@ -172,22 +186,23 @@ def delete_local_account_limit(account, rse, issuer):
     return account_limit_core.delete_local_account_limit(account=account, rse_id=rse_id)
 
 
-def delete_global_account_limit(account, rse_expression, issuer):
+def delete_global_account_limit(account, rse_expression, issuer, vo='def'):
     """
     Delete a global account limit..
 
     :param account:        The account name.
     :param rse_expression: The rse expression.
     :param issuer:         The issuer account_core.
+    :param vo:             The VO to act on.
 
     :returns: True if successful; False otherwise.
     """
 
     kwargs = {'account': account, 'rse_expression': rse_expression}
-    if not rucio.api.permission.has_permission(issuer=issuer, action='delete_global_account_limit', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='delete_global_account_limit', kwargs=kwargs):
         raise rucio.common.exception.AccessDenied('Account %s can not delete global account limits.' % (issuer))
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
     if not account_exists(account=account):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
@@ -195,13 +210,14 @@ def delete_global_account_limit(account, rse_expression, issuer):
     return account_limit_core.delete_global_account_limit(account=account, rse_expression=rse_expression)
 
 
-def get_local_account_usage(account, rse, issuer):
+def get_local_account_usage(account, rse, issuer, vo='def'):
     """
     Get the account usage and connect it with (if available) the account limits of the account.
 
     :param account:  The account to read.
     :param rse:      The rse to read (If none, get all).
     :param issuer:   The issuer account.
+    :param vo:       The VO to act on.
 
     :returns:        List of dicts {'rse_id', 'bytes_used', 'files_used', 'bytes_limit'}
     """
@@ -209,12 +225,12 @@ def get_local_account_usage(account, rse, issuer):
     rse_id = None
 
     if rse:
-        rse_id = get_rse_id(rse=rse)
+        rse_id = get_rse_id(rse=rse, vo=vo)
     kwargs = {'account': account, 'rse': rse, 'rse_id': rse_id}
-    if not rucio.api.permission.has_permission(issuer=issuer, action='get_local_account_usage', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='get_local_account_usage', kwargs=kwargs):
         raise rucio.common.exception.AccessDenied('Account %s can not list account usage.' % (issuer))
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
     if not account_exists(account=account):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
@@ -222,24 +238,25 @@ def get_local_account_usage(account, rse, issuer):
     return [api_update_return_dict(d) for d in account_limit_core.get_local_account_usage(account=account, rse_id=rse_id)]
 
 
-def get_global_account_usage(account, rse_expression, issuer):
+def get_global_account_usage(account, rse_expression, issuer, vo='def'):
     """
     Get the account usage and connect it with (if available) the account limits of the account.
 
     :param account:         The account to read.
     :param rse_expression:  The rse expression to read (If none, get all).
     :param issuer:          The issuer account.
+    :param vo:              The VO to act on.
 
     :returns:        List of dicts {'rse_id', 'bytes_used', 'files_used', 'bytes_limit'}
     """
 
     kwargs = {'account': account, 'rse_expression': rse_expression}
-    if not rucio.api.permission.has_permission(issuer=issuer, action='get_global_account_usage', kwargs=kwargs):
+    if not rucio.api.permission.has_permission(issuer=issuer, vo=vo, action='get_global_account_usage', kwargs=kwargs):
         raise rucio.common.exception.AccessDenied('Account %s can not list global account usage.' % (issuer))
 
-    account = InternalAccount(account)
+    account = InternalAccount(account, vo=vo)
 
     if not account_exists(account=account):
         raise rucio.common.exception.AccountNotFound('Account %s does not exist' % (account))
 
-    return account_limit_core.get_global_account_usage(account=account, rse_expression=rse_expression)
+    return [api_update_return_dict(d) for d in account_limit_core.get_global_account_usage(account=account, rse_expression=rse_expression)]
