@@ -1,25 +1,32 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# -*- coding: utf-8 -*-
+# Copyright 2012-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
-# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2012
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012, 2017
-# - Tomas Kouba, <tomas.kouba@cern.ch>, 2014
-# - Thomas Beermann, <thomas.beermann@cern.ch>, 2014
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2020
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2015
+# - Tomáš Kouba <tomas.kouba@cern.ch>, 2014
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2014
+# - Martin Barisits <martin.barisits@cern.ch>, 2017
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
-#
-# PY3K COMPATIBLE
 
 """
 Interface for identity abstraction layer
 """
 
-
 from rucio.api import permission
 from rucio.common import exception
 from rucio.common.types import InternalAccount
@@ -36,7 +43,7 @@ def add_identity(identity_key, id_type, email, password=None):
     :param email: The Email address associated with the identity.
     :param password: If type==userpass, this sets the password.
     """
-    return identity.add_identity(identity_key, IdentityType.from_sym(id_type), email, password=password)
+    return identity.add_identity(identity_key, IdentityType[id_type.upper()], email, password=password)
 
 
 def del_identity(identity_key, id_type, issuer, vo='def'):
@@ -47,7 +54,7 @@ def del_identity(identity_key, id_type, issuer, vo='def'):
     :param issuer: The issuer account.
     :param vo: the VO of the issuer.
     """
-    id_type = IdentityType.from_sym(id_type)
+    id_type = IdentityType[id_type.upper()]
     kwargs = {'accounts': identity.list_accounts_for_identity(identity_key, id_type)}
     if not permission.has_permission(issuer=issuer, vo=vo, action='del_identity', kwargs=kwargs):
         raise exception.AccessDenied('Account %s can not delete identity' % (issuer))
@@ -74,7 +81,7 @@ def add_account_identity(identity_key, id_type, account, email, issuer, default=
 
     account = InternalAccount(account, vo=vo)
 
-    return identity.add_account_identity(identity=identity_key, type=IdentityType.from_sym(id_type), default=default, email=email, account=account, password=password)
+    return identity.add_account_identity(identity=identity_key, type=IdentityType[id_type.upper()], default=default, email=email, account=account, password=password)
 
 
 def del_account_identity(identity_key, id_type, account, issuer, vo='def'):
@@ -93,7 +100,7 @@ def del_account_identity(identity_key, id_type, account, issuer, vo='def'):
 
     account = InternalAccount(account, vo=vo)
 
-    return identity.del_account_identity(identity_key, IdentityType.from_sym(id_type), account)
+    return identity.del_account_identity(identity_key, IdentityType[id_type.upper()], account)
 
 
 def list_identities(**kwargs):
@@ -112,7 +119,7 @@ def get_default_account(identity_key, id_type):
     :param identity_key: The identity key name. For example x509 DN, or a username.
     :param id_type: The type of the authentication (x509, gss, userpass, ssh, saml).
     """
-    account = identity.get_default_account(identity_key, IdentityType.from_sym(id_type))
+    account = identity.get_default_account(identity_key, IdentityType[id_type.upper()])
     return account.external
 
 
@@ -125,5 +132,5 @@ def list_accounts_for_identity(identity_key, id_type):
 
     returns: A list of all accounts for the identity.
     """
-    accounts = identity.list_accounts_for_identity(identity_key, IdentityType.from_sym(id_type))
+    accounts = identity.list_accounts_for_identity(identity_key, IdentityType[id_type.upper()])
     return [account.external for account in accounts]
