@@ -1,4 +1,5 @@
-# Copyright 2013-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2013-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,47 +14,44 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2013-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2019
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2019
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
 # - Yun-Pin Sun <winter0128@gmail.com>, 2013
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2013-2018
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2014-2015
-# - Wen Guan <wguan.icedew@gmail.com>, 2015
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2013-2021
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2014-2015
+# - Wen Guan <wen.guan@cern.ch>, 2015
+# - asket <asket.agarwal96@gmail.com>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Tobias Wegner <twegner@cern.ch>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
+# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2020
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Luc Goossens <luc.goossens@cern.ch>, 2020
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2019 - 2020
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 # - Vivek Nigam <viveknigam.nigam3@gmail.com>, 2020
-#
-# PY3K COMPATIBLE
 
 import logging
 import random
-import sys
-
 from datetime import datetime, timedelta
+from enum import Enum
 from hashlib import md5
 from re import match
-from six import string_types
 
+from six import string_types
 from sqlalchemy import and_, or_, exists
 from sqlalchemy.exc import DatabaseError, IntegrityError
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.sql import not_, func
 from sqlalchemy.sql.expression import bindparam, case, select, true
 
-import rucio.core.rule
 import rucio.core.replica  # import add_replicas
-
+import rucio.core.rule
 from rucio.common import exception
-from rucio.common.config import config_get
 from rucio.common.utils import str_to_date, is_archive, chunks
 from rucio.core import did_meta_plugins, config as config_core
 from rucio.core.message import add_message
@@ -61,16 +59,8 @@ from rucio.core.monitor import record_timer_block, record_counter
 from rucio.core.naming_convention import validate_name
 from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.constants import DIDType, DIDReEvaluation, DIDAvailability, RuleState
-from rucio.db.sqla.enum import EnumSymbol
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 
 @read_session
 def list_expired_dids(worker_number=None, total_workers=None, limit=None, session=None):
@@ -159,7 +149,7 @@ def add_dids(dids, account, session=None):
             try:
 
                 if isinstance(did['type'], string_types):
-                    did['type'] = DIDType.from_sym(did['type'])
+                    did['type'] = DIDType[did['type']]
 
                 if did['type'] == DIDType.FILE:
                     raise exception.UnsupportedOperation("Only collection (dataset/container) can be registered." % locals())
@@ -173,12 +163,13 @@ def add_dids(dids, account, session=None):
                 new_did = models.DataIdentifier(scope=did['scope'], name=did['name'], account=did.get('account') or account,
                                                 did_type=did['type'], monotonic=did.get('statuses', {}).get('monotonic', False),
                                                 is_open=True, expired_at=expired_at)
-                # Add metadata
-                for key in did.get('meta', {}):
-                    new_did.update({key: did['meta'][key]})
 
                 new_did.save(session=session, flush=False)
 
+                if 'meta' in did and did['meta']:
+                    # Add metadata
+                    set_metadata_bulk(scope=did['scope'], name=did['name'], meta=did['meta'], recursive=False, session=session)
+
                 if did.get('dids', None):
                     attach_dids(scope=did['scope'], name=did['name'], dids=did['dids'],
                                 account=account, rse_id=did.get('rse_id'), session=session)
@@ -564,7 +555,7 @@ def attach_dids_to_dids(attachments, account, ignore_duplicate=False, session=No
 
 
 @transactional_session
-def delete_dids(dids, account, expire_rules=False, session=None):
+def delete_dids(dids, account, expire_rules=False, session=None, logger=logging.log):
     """
     Delete data identifiers
 
@@ -574,6 +565,7 @@ def delete_dids(dids, account, expire_rules=False, session=None):
                           the method returns normally, but a did was not deleted; This trusts in the fact that the undertaker will retry an
                           expired did.
     :param session:       The database session in use.
+    :param logger:        Optional decorated logger that can be passed from the calling daemons or servers.
     """
     rule_id_clause, content_clause = [], []
     parent_content_clause, did_clause = [], []
@@ -583,7 +575,7 @@ def delete_dids(dids, account, expire_rules=False, session=None):
     metadata_to_delete = []
 
     for did in dids:
-        logging.info('Removing did %(scope)s:%(name)s (%(did_type)s)' % did)
+        logger(logging.INFO, 'Removing did %(scope)s:%(name)s (%(did_type)s)' % did)
         if did['did_type'] == DIDType.FILE:
             file_clause.append(and_(models.DataIdentifier.scope == did['scope'], models.DataIdentifier.name == did['name']))
         else:
@@ -639,7 +631,7 @@ def delete_dids(dids, account, expire_rules=False, session=None):
                                                                                                                               models.ReplicationRule.locks_ok_cnt,
                                                                                                                               models.ReplicationRule.locks_replicating_cnt,
                                                                                                                               models.ReplicationRule.locks_stuck_cnt).filter(or_(*rule_id_clause)):
-                logging.debug('Removing rule %s for did %s:%s on RSE-Expression %s' % (str(rule_id), scope, name, rse_expression))
+                logger(logging.DEBUG, 'Removing rule %s for did %s:%s on RSE-Expression %s' % (str(rule_id), scope, name, rse_expression))
 
                 # Propagate purge_replicas from did to rules
                 if (scope, name) in not_purge_replicas:
@@ -695,7 +687,7 @@ def delete_dids(dids, account, expire_rules=False, session=None):
     # remove data identifier
     if existing_parent_dids:
         # Exit method early to give Judge time to remove locks (Otherwise, due to foreign keys, did removal does not work
-        logging.debug('Leaving delete_dids early for Judge-Evaluator checks')
+        logger(logging.DEBUG, 'Leaving delete_dids early for Judge-Evaluator checks')
         return
 
     if did_clause:
@@ -836,8 +828,8 @@ def list_new_dids(did_type, thread=None, total_threads=None, chunk_size=1000, se
 
     if did_type:
         if isinstance(did_type, string_types):
-            query = query.filter_by(did_type=DIDType.from_sym(did_type))
-        elif isinstance(did_type, EnumSymbol):
+            query = query.filter_by(did_type=DIDType[did_type])
+        elif isinstance(did_type, Enum):
             query = query.filter_by(did_type=did_type)
 
     query = filter_thread_work(session=session, query=query, total_threads=total_threads, thread_id=thread, hash_variable='name')
@@ -1241,6 +1233,20 @@ def set_metadata(scope, name, key, value, type=None, did=None,
     did_meta_plugins.set_metadata(scope=scope, name=name, key=key, value=value, recursive=recursive, session=session)
 
 
+@transactional_session
+def set_metadata_bulk(scope, name, meta, recursive=False, session=None):
+    """
+    Add metadata to data identifier.
+
+    :param scope: The scope name.
+    :param name: The data identifier name.
+    :param meta: the key-values.
+    :param recursive: Option to propagate the metadata change to content.
+    :param session: The database session in use.
+    """
+    did_meta_plugins.set_metadata_bulk(scope=scope, name=name, meta=meta, recursive=recursive, session=session)
+
+
 @read_session
 def get_metadata(scope, name, plugin='DID_COLUMN', session=None):
     """
@@ -1466,7 +1472,7 @@ def list_dids(scope, filters, type='collection', ignore_case=False, limit=None,
         for scope, name, did_type, bytes, length in query.yield_per(5):
             yield {'scope': scope,
                    'name': name,
-                   'did_type': str(did_type),
+                   'did_type': did_type.name,
                    'bytes': bytes,
                    'length': length}
     else:
