@@ -1,4 +1,5 @@
-# Copyright 2014-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2014-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +14,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2019
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2020
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2017
 # - Martin Barisits <martin.barisits@cern.ch>, 2014-2019
 # - Robert Illingworth <illingwo@fnal.gov>, 2018
@@ -21,8 +22,6 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
 
 import json
 
@@ -107,18 +106,25 @@ def retrieve_messages(bulk=1000, thread=None, total_threads=None, event_type=Non
         # Step 1:
         # MySQL does not support limits in nested queries, limit on the outer query instead.
         # This is not as performant, but the best we can get from MySQL.
+        # FIXME: SQLAlchemy generates wrong nowait MySQL8 statement for MySQL5
+        #        Remove once this is resolved in SQLAlchemy
         if session.bind.dialect.name == 'mysql':
             subquery = subquery.order_by(Message.created_at)
+            query = session.query(Message.id,
+                                  Message.created_at,
+                                  Message.event_type,
+                                  Message.payload,
+                                  Message.services)\
+                           .filter(Message.id.in_(subquery))
         else:
             subquery = subquery.order_by(Message.created_at).limit(bulk)
-
-        query = session.query(Message.id,
-                              Message.created_at,
-                              Message.event_type,
-                              Message.payload,
-                              Message.services)\
-            .filter(Message.id.in_(subquery))\
-            .with_for_update(nowait=True)
+            query = session.query(Message.id,
+                                  Message.created_at,
+                                  Message.event_type,
+                                  Message.payload,
+                                  Message.services)\
+                           .filter(Message.id.in_(subquery))\
+                           .with_for_update(nowait=True)
 
         # Step 2:
         # MySQL does not support limits in nested queries, limit on the outer query instead.
