@@ -1,49 +1,63 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# -*- coding: utf-8 -*-
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
+# you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2013-2015, 2017
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2015-2017
-# - Martin Barisits, <martin.barisits@cern.ch>, 2014-2017
-# - Wen Guan, <wen.guan@cern.ch>, 2014-2016
-# - Joaquin Bogado, <jbogadog@cern.ch>, 2016
-# - Thomas Beermann, <thomas.beermann@cern.ch>, 2016
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2017-2020
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White, <bjwhite@fnal.gov>, 2019
-#
-# PY3K COMPATIBLE
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2021
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2017
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2020
+# - Wen Guan <wen.guan@cern.ch>, 2014-2016
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2019
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2016
+# - Joaquin Bogado <jbogadog@cern.ch>, 2017
+# - Igor Mandrichenko <rucio@fermicloud055.fnal.gov>, 2018
+# - Robert Illingworth <illingwo@fnal.gov>, 2018
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 import datetime
 import json
 import logging
 import time
 import traceback
+from typing import TYPE_CHECKING
 
 from six import string_types
-
 from sqlalchemy import and_, or_, func, update
 from sqlalchemy.exc import IntegrityError
-# from sqlalchemy.sql import tuple_
 from sqlalchemy.sql.expression import asc, false, true
 
+from rucio.common.config import config_get_bool
 from rucio.common.exception import RequestNotFound, RucioException, UnsupportedOperation, ConfigNotFound
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid, chunks, get_parsed_throttler_mode
 from rucio.core.config import get
 from rucio.core.message import add_message
 from rucio.core.monitor import record_counter, record_timer
-from rucio.core.rse import get_rse_name, get_rse_transfer_limits, get_rse_vo
+from rucio.core.rse import get_rse_name, get_rse_vo, get_rse_transfer_limits
 from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.constants import RequestState, RequestType, FTSState, ReplicaState, LockState, RequestErrMsg
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 from rucio.transfertool.fts3 import FTS3Transfertool
 
+if TYPE_CHECKING:
+    from typing import List, Tuple, Iterable, Iterator, Optional
+    from sqlalchemy.orm import Session
+
 """
 The core request.py is specifically for handling requests.
 Requests accessed by external_id (So called transfers), are covered in the core transfer.py
@@ -71,13 +85,14 @@ def should_retry_request(req, retry_protocol_mismatches):
 
 
 @transactional_session
-def requeue_and_archive(request, retry_protocol_mismatches=False, session=None):
+def requeue_and_archive(request, retry_protocol_mismatches=False, session=None, logger=logging.log):
     """
     Requeue and archive a failed request.
     TODO: Multiple requeue.
 
     :param request:     Original request.
     :param session:     Database session to use.
+    :param logger:      Optional decorated logger that can be passed from the calling daemons or servers.
     """
 
     record_counter('core.request.requeue_request')
@@ -105,7 +120,7 @@ def requeue_and_archive(request, retry_protocol_mismatches=False, session=None):
                         else:
                             new_req['sources'][i]['ranking'] -= 1
                         new_req['sources'][i]['is_using'] = False
-            queue_requests([new_req], session=session)
+            queue_requests([new_req], session=session, logger=logger)
             return new_req
     else:
         raise RequestNotFound
@@ -113,20 +128,22 @@ def requeue_and_archive(request, retry_protocol_mismatches=False, session=None):
 
 
 @transactional_session
-def queue_requests(requests, session=None):
+def queue_requests(requests, session=None, logger=logging.log):
     """
     Submit transfer requests on destination RSEs for data identifiers.
 
     :param requests:  List of dictionaries containing request metadata.
     :param session:   Database session to use.
+    :param logger:    Optional decorated logger that can be passed from the calling daemons or servers.
     :returns:         List of Request-IDs as 32 character hex strings.
     """
     record_counter('core.request.queue_requests')
 
-    logging.debug("queue requests")
+    logger(logging.DEBUG, "queue requests")
 
     request_clause = []
-    transfer_limits, rses = get_rse_transfer_limits(session=session), {}
+    rses = {}
+    preparer_enabled = config_get_bool('conveyor', 'use_preparer', raise_exception=False, default=False)
     for req in requests:
 
         if isinstance(req['attributes'], string_types):
@@ -157,43 +174,14 @@ def queue_requests(requests, session=None):
             for request in query_existing_requests:
                 existing_requests.append(request)
 
-    # Temporary disabled
-    source_rses = {}
-    # request_scopes_names = [(request['scope'], request['name']) for request in requests]
-    # for chunked_requests in chunks(request_scopes_names, 50):
-    #    results = session.query(models.RSEFileAssociation.scope, models.RSEFileAssociation.name, models.RSEFileAssociation.rse_id, models.Distance.dest_rse_id, models.Distance.ranking)\
-    #                     .filter(tuple_(models.RSEFileAssociation.scope, models.RSEFileAssociation.name).in_(chunked_requests))\
-    #                     .join(models.Distance, models.Distance.src_rse_id == models.RSEFileAssociation.rse_id)\
-    #                     .all()
-    #    for result in results:
-    #        scope = result[0]
-    #        name = result[1]
-    #        src_rse_id = result[2]
-    #        dest_rse_id = result[3]
-    #        distance = result[4]
-    #        if scope not in source_rses:
-    #            source_rses[scope] = {}
-    #        if name not in source_rses[scope]:
-    #            source_rses[scope][name] = {}
-    #        if dest_rse_id not in source_rses[scope][name]:
-    #            source_rses[scope][name][dest_rse_id] = {}
-    #        if src_rse_id not in source_rses[scope][name][dest_rse_id]:
-    #            source_rses[scope][name][dest_rse_id][src_rse_id] = distance
-
-    try:
-        throttler_mode = get('throttler', 'mode', default=None, use_cache=False, session=session)
-        direction, all_activities = get_parsed_throttler_mode(throttler_mode)
-    except ConfigNotFound:
-        throttler_mode = None
-
     new_requests, sources, messages = [], [], []
     for request in requests:
         dest_rse_name = get_rse_name(rse_id=request['dest_rse_id'], session=session)
         if req['request_type'] == RequestType.TRANSFER and (request['scope'], request['name'], request['dest_rse_id']) in existing_requests:
-            logging.warn('Request TYPE %s for DID %s:%s at RSE %s exists - ignoring' % (request['request_type'],
-                                                                                        request['scope'],
-                                                                                        request['name'],
-                                                                                        dest_rse_name))
+            logger(logging.WARNING, 'Request TYPE %s for DID %s:%s at RSE %s exists - ignoring' % (request['request_type'],
+                                                                                                   request['scope'],
+                                                                                                   request['name'],
+                                                                                                   dest_rse_name))
             continue
 
         def temp_serializer(obj):
@@ -201,38 +189,13 @@ def queue_requests(requests, session=None):
                 return obj.internal
             raise TypeError('Could not serialise object %r' % obj)
 
-        source_rse_id = request.get('source_rse_id')
-        if not source_rse_id:
-            try:
-                source_rses_of_request = source_rses[request['scope']][request['name']][request['dest_rse_id']]
-                source_rse_id = min(source_rses_of_request, key=source_rses_of_request.get)
-            except KeyError:
-                pass
-        activity_limit = transfer_limits.get(request['attributes']['activity'], {})
-        all_activities_limit = transfer_limits.get('all_activities', {})
-        limit_found = False
-        if throttler_mode:
-            if direction == 'source':
-                if all_activities:
-                    if all_activities_limit.get(source_rse_id):
-                        limit_found = True
-                else:
-                    if activity_limit.get(source_rse_id):
-                        limit_found = True
-            elif direction == 'destination':
-                if all_activities:
-                    if all_activities_limit.get(request['dest_rse_id']):
-                        limit_found = True
-                else:
-                    if activity_limit.get(request['dest_rse_id']):
-                        limit_found = True
-        request['state'] = RequestState.WAITING if limit_found else RequestState.QUEUED
+        request['state'] = RequestState.PREPARING if preparer_enabled else RequestState.QUEUED
 
         new_request = {'request_type': request['request_type'],
                        'scope': request['scope'],
                        'name': request['name'],
                        'dest_rse_id': request['dest_rse_id'],
-                       'source_rse_id': source_rse_id,
+                       'source_rse_id': request.get('source_rse_id', None),
                        'attributes': json.dumps(request['attributes'], default=temp_serializer),
                        'state': request['state'],
                        'rule_id': request['rule_id'],
@@ -264,17 +227,18 @@ def queue_requests(requests, session=None):
                                 'is_using': source['is_using']})
 
         if request['request_type']:
-            transfer_status = '%s-%s' % (request['request_type'], request['state'])
+            transfer_status = '%s-%s' % (request['request_type'].name, request['state'].name)
         else:
-            transfer_status = 'transfer-%s' % request['state']
+            transfer_status = 'transfer-%s' % request['state'].name
+        transfer_status = transfer_status.lower()
 
         payload = {'request-id': new_request['id'],
-                   'request-type': str(request['request_type']).lower(),
+                   'request-type': request['request_type'].name.lower(),
                    'scope': request['scope'].external,
                    'name': request['name'],
                    'dst-rse-id': request['dest_rse_id'],
                    'dst-rse': dest_rse_name,
-                   'state': str(request['state']),
+                   'state': request['state'].name.lower(),
                    'retry-count': request['retry_count'],
                    'rule-id': str(request['rule_id']),
                    'activity': request['attributes']['activity'],
@@ -284,7 +248,7 @@ def queue_requests(requests, session=None):
                    'checksum-adler': request['attributes']['adler32'],
                    'queued_at': str(datetime.datetime.utcnow())}
 
-        messages.append({'event_type': transfer_status.lower(),
+        messages.append({'event_type': transfer_status,
                          'payload': json.dumps(payload)})
 
     for requests_chunk in chunks(new_requests, 1000):
@@ -295,6 +259,7 @@ def queue_requests(requests, session=None):
 
     for messages_chunk in chunks(messages, 1000):
         session.bulk_insert_mappings(models.Message, messages_chunk)
+
     return new_requests
 
 
@@ -385,13 +350,14 @@ def get_next(request_type, state, limit=100, older_than=None, rse_id=None, activ
 
 
 @read_session
-def query_request(request_id, transfertool='fts3', session=None):
+def query_request(request_id, transfertool='fts3', session=None, logger=logging.log):
     """
     Query the status of a request.
 
     :param request_id:    Request-ID as a 32 character hex string.
     :param transfertool:  Transfertool name as a string.
     :param session:       Database session to use.
+    :param logger:        Optional decorated logger that can be passed from the calling daemons or servers.
     :returns:             Request status information as a dictionary.
     """
 
@@ -420,11 +386,11 @@ def query_request(request_id, transfertool='fts3', session=None):
         else:
             if 'job_state' not in response:
                 req_status['new_state'] = RequestState.LOST
-            elif response['job_state'] in (str(FTSState.FAILED),
-                                           str(FTSState.FINISHEDDIRTY),
-                                           str(FTSState.CANCELED)):
+            elif response['job_state'] in (str(FTSState.FAILED.name),
+                                           str(FTSState.FINISHEDDIRTY.name),
+                                           str(FTSState.CANCELED.name)):
                 req_status['new_state'] = RequestState.FAILED
-            elif response['job_state'] == str(FTSState.FINISHED):
+            elif response['job_state'] == str(FTSState.FINISHED.name):
                 req_status['new_state'] = RequestState.DONE
     else:
         raise NotImplementedError
@@ -433,7 +399,7 @@ def query_request(request_id, transfertool='fts3', session=None):
 
 
 @read_session
-def query_request_details(request_id, transfertool='fts3', session=None):
+def query_request_details(request_id, transfertool='fts3', session=None, logger=logging.log):
     """
     Query the detailed status of a request. Can also be done after the
     external transfer has finished.
@@ -441,6 +407,7 @@ def query_request_details(request_id, transfertool='fts3', session=None):
     :param request_id:    Request-ID as a 32 character hex string.
     :param transfertool:  Transfertool name as a string.
     :param session:       Database session to use.
+    :param logger:        Optional decorated logger that can be passed from the calling daemons or servers.
     :returns:             Detailed request status information as a dictionary.
     """
 
@@ -461,18 +428,19 @@ def query_request_details(request_id, transfertool='fts3', session=None):
 
 
 @transactional_session
-def set_request_state(request_id, new_state, transfer_id=None, transferred_at=None, started_at=None, staging_started_at=None, staging_finished_at=None, src_rse_id=None, err_msg=None, session=None):
+def set_request_state(request_id, new_state, transfer_id=None, transferred_at=None, started_at=None, staging_started_at=None, staging_finished_at=None, src_rse_id=None, err_msg=None, session=None, logger=logging.log):
     """
     Update the state of a request. Fails silently if the request_id does not exist.
 
-    :param request_id:   Request-ID as a 32 character hex string.
-    :param new_state:    New state as string.
-    :param transfer_id:  External transfer job id as a string.
-    :param transferred_at: Transferred at timestamp
-    :param started_at: Started at timestamp
-    :param staging_started_at: Timestamp indicating the moment the stage beggins
-    :param staging_finished_at: Timestamp indicating the moment the stage ends
-    :param session:      Database session to use.
+    :param request_id:           Request-ID as a 32 character hex string.
+    :param new_state:            New state as string.
+    :param transfer_id:          External transfer job id as a string.
+    :param transferred_at:       Transferred at timestamp
+    :param started_at:           Started at timestamp
+    :param staging_started_at:   Timestamp indicating the moment the stage beggins
+    :param staging_finished_at:  Timestamp indicating the moment the stage ends
+    :param logger:               Optional decorated logger that can be passed from the calling daemons or servers.
+    :param session:              Database session to use.
     """
 
     # TODO: Should this be a private method?
@@ -499,7 +467,7 @@ def set_request_state(request_id, new_state, transfer_id=None, transferred_at=No
             rowcount = session.query(models.Request).filter_by(id=request_id, external_id=transfer_id).update(update_items, synchronize_session=False)
         else:
             if new_state in [RequestState.FAILED, RequestState.DONE, RequestState.LOST]:
-                logging.error("Request %s should not be updated to 'Failed' or 'Done' without external transfer_id" % request_id)
+                logger(logging.ERROR, "Request %s should not be updated to 'Failed' or 'Done' without external transfer_id" % request_id)
             else:
                 rowcount = session.query(models.Request).filter_by(id=request_id).update(update_items, synchronize_session=False)
     except IntegrityError as error:
@@ -510,20 +478,21 @@ def set_request_state(request_id, new_state, transfer_id=None, transferred_at=No
 
 
 @transactional_session
-def set_requests_state(request_ids, new_state, session=None):
+def set_requests_state(request_ids, new_state, session=None, logger=logging.log):
     """
     Bulk update the state of requests. Fails silently if the request_id does not exist.
 
     :param request_ids:  List of (Request-ID as a 32 character hex string).
     :param new_state:    New state as string.
     :param session:      Database session to use.
+    :param logger:       Optional decorated logger that can be passed from the calling daemons or servers.
     """
 
     record_counter('core.request.set_requests_state')
 
     try:
         for request_id in request_ids:
-            set_request_state(request_id, new_state, session=session)
+            set_request_state(request_id, new_state, session=session, logger=logger)
     except IntegrityError as error:
         raise RucioException(error.args)
 
@@ -693,7 +662,7 @@ def archive_request(request_id, session=None):
 
 
 @transactional_session
-def cancel_request_did(scope, name, dest_rse_id, request_type=RequestType.TRANSFER, session=None):
+def cancel_request_did(scope, name, dest_rse_id, request_type=RequestType.TRANSFER, session=None, logger=logging.log):
     """
     Cancel a request based on a DID and request type.
 
@@ -702,6 +671,7 @@ def cancel_request_did(scope, name, dest_rse_id, request_type=RequestType.TRANSF
     :param dest_rse_id:   RSE id as a string.
     :param request_type:  Type of the request.
     :param session:       Database session to use.
+    :param logger:        Optional decorated logger that can be passed from the calling daemons or servers.
     """
 
     record_counter('core.request.cancel_request_did')
@@ -715,7 +685,7 @@ def cancel_request_did(scope, name, dest_rse_id, request_type=RequestType.TRANSF
                                                                      dest_rse_id=dest_rse_id,
                                                                      request_type=request_type).all()
         if not reqs:
-            logging.warn('Tried to cancel non-existant request for DID %s:%s at RSE %s' % (scope, name, get_rse_name(rse_id=dest_rse_id, session=session)))
+            logger(logging.WARNING, 'Tried to cancel non-existant request for DID %s:%s at RSE %s' % (scope, name, get_rse_name(rse_id=dest_rse_id, session=session)))
     except IntegrityError as error:
         raise RucioException(error.args)
 
@@ -728,7 +698,7 @@ def cancel_request_did(scope, name, dest_rse_id, request_type=RequestType.TRANSF
                     transfertool_map[req[2]] = FTS3Transfertool(external_host=req[2])
                 transfertool_map[req[2]].cancel(transfer_ids=[req[1]])
             except Exception as error:
-                logging.warn('Could not cancel FTS3 transfer %s on %s: %s' % (req[1], req[2], str(error)))
+                logger(logging.WARNING, 'Could not cancel FTS3 transfer %s on %s: %s' % (req[1], req[2], str(error)))
         archive_request(request_id=req[0], session=session)
 
 
@@ -1248,12 +1218,13 @@ def release_all_waiting_requests(rse_id, activity=None, account=None, direction=
 
 
 @read_session
-def update_requests_priority(priority, filter, session=None):
+def update_requests_priority(priority, filter, session=None, logger=logging.log):
     """
     Update priority of requests.
 
     :param priority:  The priority as an integer from 1 to 5.
     :param filter:    Dictionary such as {'rule_id': rule_id, 'request_id': request_id, 'older_than': time_stamp, 'activities': [activities]}.
+    :param logger:    Optional decorated logger that can be passed from the calling daemons or servers.
     """
     try:
         query = session.query(models.Request.id, models.Request.external_id, models.Request.external_host)\
@@ -1280,15 +1251,15 @@ def update_requests_priority(priority, filter, session=None):
                     transfertool_map[item[2]] = FTS3Transfertool(external_host=item[2])
                 res = transfertool_map[item[2]].update_priority(transfer_id=item[1], priority=priority)
             except Exception:
-                logging.debug("Failed to boost request %s priority: %s" % (item[0], traceback.format_exc()))
+                logger(logging.DEBUG, "Failed to boost request %s priority: %s" % (item[0], traceback.format_exc()))
             else:
-                logging.debug("Update request %s priority to %s: %s" % (item[0], priority, res['http_message']))
+                logger(logging.DEBUG, "Update request %s priority to %s: %s" % (item[0], priority, res['http_message']))
     except IntegrityError as error:
         raise RucioException(error.args)
 
 
 @transactional_session
-def update_request_state(response, logging_prepend_str=None, session=None):
+def update_request_state(response, session=None, logger=logging.log):
     """
     Used by poller and consumer to update the internal state of requests,
     after the response by the external transfertool.
@@ -1296,12 +1267,10 @@ def update_request_state(response, logging_prepend_str=None, session=None):
     :param response:              The transfertool response dictionary, retrieved via request.query_request().
     :param logging_prepend_str:   String to prepend to the logging
     :param session:               The database session to use.
+    :param logger:                Optional decorated logger that can be passed from the calling daemons or servers.
     :returns commit_or_rollback:  Boolean.
     """
 
-    prepend_str = ''
-    if logging_prepend_str:
-        prepend_str = logging_prepend_str
     try:
         if not response['new_state']:
             __touch_request(response['request_id'], session=session)
@@ -1312,7 +1281,7 @@ def update_request_state(response, logging_prepend_str=None, session=None):
                 response['submitted_at'] = request.get('submitted_at', None)
                 response['external_host'] = request['external_host']
                 transfer_id = response['transfer_id'] if 'transfer_id' in response else None
-                logging.info(prepend_str + 'UPDATING REQUEST %s FOR TRANSFER %s STATE %s' % (str(response['request_id']), transfer_id, str(response['new_state'])))
+                logger(logging.INFO, 'UPDATING REQUEST %s FOR TRANSFER %s STATE %s' % (str(response['request_id']), transfer_id, str(response['new_state'])))
 
                 job_m_replica = response.get('job_m_replica', None)
                 src_url = response.get('src_url', None)
@@ -1326,12 +1295,12 @@ def update_request_state(response, logging_prepend_str=None, session=None):
                     try:
                         src_rse_name, src_rse_id = __get_source_rse(response['request_id'], src_url, session=session)
                     except Exception:
-                        logging.warn(prepend_str + 'Cannot get correct RSE for source url: %s(%s)' % (src_url, traceback.format_exc()))
+                        logger(logging.WARNING, 'Cannot get correct RSE for source url: %s(%s)' % (src_url, traceback.format_exc()))
                         src_rse_name = None
                     if src_rse_name and src_rse_name != src_rse:
                         response['src_rse'] = src_rse_name
                         response['src_rse_id'] = src_rse_id
-                        logging.debug(prepend_str + 'Correct RSE: %s for source surl: %s' % (src_rse_name, src_url))
+                        logger(logging.DEBUG, 'Correct RSE: %s for source surl: %s' % (src_rse_name, src_url))
                 err_msg = get_transfer_error(response['new_state'], response['reason'] if 'reason' in response else None)
 
                 set_request_state(response['request_id'],
@@ -1343,24 +1312,25 @@ def update_request_state(response, logging_prepend_str=None, session=None):
                                   transferred_at=transferred_at,
                                   src_rse_id=src_rse_id,
                                   err_msg=err_msg,
-                                  session=session)
+                                  session=session,
+                                  logger=logger)
 
                 add_monitor_message(request, response, session=session)
                 return True
             elif not request:
-                logging.debug(prepend_str + "Request %s doesn't exist, will not update" % (response['request_id']))
+                logger(logging.DEBUG, "Request %s doesn't exist, will not update" % (response['request_id']))
                 return False
             elif request['external_id'] != response['transfer_id']:
-                logging.warning(prepend_str + "Response %s with transfer id %s is different from the request transfer id %s, will not update" % (response['request_id'], response['transfer_id'], request['external_id']))
+                logger(logging.WARNING, "Response %s with transfer id %s is different from the request transfer id %s, will not update" % (response['request_id'], response['transfer_id'], request['external_id']))
                 return False
             else:
-                logging.debug(prepend_str + "Request %s is already in %s state, will not update" % (response['request_id'], response['new_state']))
+                logger(logging.DEBUG, "Request %s is already in %s state, will not update" % (response['request_id'], response['new_state']))
                 return False
     except UnsupportedOperation as error:
-        logging.warning(prepend_str + "Request %s doesn't exist - Error: %s" % (response['request_id'], str(error).replace('\n', '')))
+        logger(logging.WARNING, "Request %s doesn't exist - Error: %s" % (response['request_id'], str(error).replace('\n', '')))
         return False
     except Exception:
-        logging.critical(prepend_str + traceback.format_exc())
+        logger(logging.CRITICAL, "Exception", exc_info=True)
 
 
 @read_session
@@ -1374,9 +1344,9 @@ def add_monitor_message(request, response, session=None):
     """
 
     if request['request_type']:
-        transfer_status = '%s-%s' % (request['request_type'], response['new_state'])
+        transfer_status = '%s-%s' % (request['request_type'].name, response['new_state'].name)
     else:
-        transfer_status = 'transfer-%s' % (response['new_state'])
+        transfer_status = 'transfer-%s' % (response['new_state'].name)
     transfer_status = transfer_status.lower()
 
     activity = response.get('activity', None)
@@ -1485,7 +1455,7 @@ def __touch_request(request_id, session=None):
 
 
 @read_session
-def __get_source_rse(request_id, src_url, session=None):
+def __get_source_rse(request_id, src_url, session=None, logger=logging.log):
     """
     Based on a request, scope, name and src_url extract the source rse name and id.
 
@@ -1494,6 +1464,7 @@ def __get_source_rse(request_id, src_url, session=None):
     :param name:        The name of the request file.
     :param src_url:     The src_url of the request.
     :param session:     The database session to use.
+    :param logger:      Optional decorated logger that can be passed from the calling daemons or servers.
     """
 
     try:
@@ -1505,13 +1476,13 @@ def __get_source_rse(request_id, src_url, session=None):
             if source['url'] == src_url:
                 src_rse_id = source['rse_id']
                 src_rse_name = get_rse_name(src_rse_id, session=session)
-                logging.debug("Find rse name %s for %s" % (src_rse_name, src_url))
+                logger(logging.DEBUG, "Find rse name %s for %s" % (src_rse_name, src_url))
                 return src_rse_name, src_rse_id
         # cannot find matched surl
-        logging.warn('Cannot get correct RSE for source url: %s' % (src_url))
+        logger(logging.WARNING, 'Cannot get correct RSE for source url: %s' % (src_url))
         return None, None
     except Exception:
-        logging.error('Cannot get correct RSE for source url: %s(%s)' % (src_url, traceback.format_exc()))
+        logger(logging.ERROR, 'Cannot get correct RSE for source url: %s' % (src_url), exc_info=True)
         return None, None
 
 
@@ -1530,3 +1501,81 @@ def list_requests(src_rse_ids, dst_rse_ids, states=[RequestState.WAITING], sessi
                                                  models.Request.dest_rse_id.in_(dst_rse_ids))
     for request in query.yield_per(500):
         yield request
+
+
+# important column indices from __list_transfer_requests_and_source_replicas
+request_id_col = 0
+activity_col = 7
+dest_rse_id_col = 10
+source_rse_id_col = 12
+distance_col = 20
+
+
+@transactional_session
+def preparer_update_requests(source_iter: "Iterable[Tuple]", session: "Optional[Session]" = None) -> int:
+    count = 0
+    for req_source in source_iter:
+        new_state = __throttler_request_state(
+            activity=req_source[activity_col],
+            source_rse_id=req_source[source_rse_id_col],
+            dest_rse_id=req_source[dest_rse_id_col],
+            session=session,
+        )
+        session.query(models.Request).filter_by(id=req_source[request_id_col]).update({
+            models.Request.state: new_state,
+            models.Request.source_rse_id: req_source[source_rse_id_col],
+        }, synchronize_session=False)
+        count += 1
+    return count
+
+
+def __throttler_request_state(activity, source_rse_id, dest_rse_id, session: "Optional[Session]" = None) -> RequestState:
+    """
+    Takes request attributes to return a new state for the request
+    based on throttler settings. Always returns QUEUED,
+    if the throttler mode is not set.
+    """
+    try:
+        throttler_mode = get('throttler', 'mode', default=None, use_cache=False, session=session)
+    except ConfigNotFound:
+        throttler_mode = None
+
+    limit_found = False
+    if throttler_mode:
+        transfer_limits = get_rse_transfer_limits(session=session)
+        activity_limit = transfer_limits.get(activity, {})
+        all_activities_limit = transfer_limits.get('all_activities', {})
+        direction, all_activities = get_parsed_throttler_mode(throttler_mode)
+        if direction == 'source':
+            if all_activities:
+                if all_activities_limit.get(source_rse_id):
+                    limit_found = True
+            else:
+                if activity_limit.get(source_rse_id):
+                    limit_found = True
+        elif direction == 'destination':
+            if all_activities:
+                if all_activities_limit.get(dest_rse_id):
+                    limit_found = True
+            else:
+                if activity_limit.get(dest_rse_id):
+                    limit_found = True
+
+    return RequestState.WAITING if limit_found else RequestState.QUEUED
+
+
+def minimum_distance_requests(req_sources: "List[Tuple]") -> "Iterator[Tuple]":
+    # sort by Request.id, should be pretty quick since the database should return it this way (tim sort)
+    req_sources.sort(key=lambda t: t[request_id_col])
+
+    # req_sources must be non-empty and sorted by ids at this point, see above
+    cur_request_id = req_sources[0][request_id_col]
+    shortest_item = req_sources[0]
+    for idx in range(1, len(req_sources)):
+        if cur_request_id != req_sources[idx][request_id_col]:
+            yield shortest_item
+            cur_request_id = req_sources[idx][request_id_col]
+            shortest_item = req_sources[idx]
+        elif req_sources[idx][distance_col] < shortest_item[distance_col]:
+            shortest_item = req_sources[idx]
+    yield shortest_item
