@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2021
 # - Martin Barisits <martin.barisits@cern.ch>, 2017-2020
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2017
 # - Igor Mandrichenko <rucio@fermicloud055.fnal.gov>, 2018
@@ -32,9 +32,8 @@
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Nick Smith <nick.smith@cern.ch>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
-
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
 from __future__ import division
 
@@ -45,7 +44,7 @@ import json
 import logging
 import re
 import time
-import traceback
+from typing import TYPE_CHECKING
 
 from dogpile.cache import make_region
 from dogpile.cache.api import NoValue
@@ -54,14 +53,14 @@ from sqlalchemy.exc import IntegrityError
 from sqlalchemy.sql.expression import false
 
 from rucio.common import constants
+from rucio.common.config import config_get
+from rucio.common.constants import SUPPORTED_PROTOCOLS
 from rucio.common.exception import (InvalidRSEExpression, NoDistance,
                                     RequestNotFound, RSEProtocolNotSupported,
                                     RucioException, UnsupportedOperation)
-from rucio.common.config import config_get
 from rucio.common.rse_attributes import get_rse_attributes
 from rucio.common.types import InternalAccount
 from rucio.common.utils import construct_surl
-from rucio.common.constants import SUPPORTED_PROTOCOLS
 from rucio.core import did, message as message_core, request as request_core
 from rucio.core.config import get as core_config_get
 from rucio.core.monitor import record_counter, record_timer
@@ -76,6 +75,8 @@ from rucio.db.sqla.session import read_session, transactional_session
 from rucio.rse import rsemanager as rsemgr
 from rucio.transfertool.fts3 import FTS3Transfertool
 
+if TYPE_CHECKING:
+    from typing import List, Tuple
 
 # Extra modules: Only imported if available
 EXTRA_MODULES = {'globus_sdk': False}
@@ -108,13 +109,14 @@ REQUEST_OIDC_AUDIENCE = config_get('conveyor', 'request_oidc_audience', False, '
 WEBDAV_TRANSFER_MODE = config_get('conveyor', 'webdav_transfer_mode', False, None)
 
 
-def submit_bulk_transfers(external_host, files, transfertool='fts3', job_params={}, timeout=None, user_transfer_job=False):
+def submit_bulk_transfers(external_host, files, transfertool='fts3', job_params={}, timeout=None, user_transfer_job=False, logger=logging.log):
     """
     Submit transfer request to a transfertool.
     :param external_host:  External host name as string
     :param files:          List of Dictionary containing request file.
     :param transfertool:   Transfertool as a string.
     :param job_params:     Metadata key/value pairs for all files as a dictionary.
+    :param logger:         Optional decorated logger that can be passed from the calling daemons or servers.
     :returns:              Transfertool external ID.
     """
 
@@ -141,23 +143,23 @@ def submit_bulk_transfers(external_host, files, transfertool='fts3', job_params=
         use_oidc = job_params.get('use_oidc', False)
         transfer_token = None
         if use_oidc:
-            logging.debug('OAuth2/OIDC available at RSEs')
+            logger(logging.DEBUG, 'OAuth2/OIDC available at RSEs')
             account = job_params.get('account', None)
             getadmintoken = False
             if ALLOW_USER_OIDC_TOKENS is False:
                 getadmintoken = True
-            logging.debug('Attempting to get a token for account %s. Admin token option set to %s' % (account, getadmintoken))
+            logger(logging.DEBUG, 'Attempting to get a token for account %s. Admin token option set to %s' % (account, getadmintoken))
             # find the appropriate OIDC token and exchange it (for user accounts) if necessary
             token_dict = get_token_for_account_operation(account, req_audience=REQUEST_OIDC_AUDIENCE, req_scope=REQUEST_OIDC_SCOPE, admin=getadmintoken)
             if token_dict is not None:
-                logging.debug('Access token has been granted.')
+                logger(logging.DEBUG, 'Access token has been granted.')
                 if 'token' in token_dict:
-                    logging.debug('Access token used as transfer token.')
+                    logger(logging.DEBUG, 'Access token used as transfer token.')
                     transfer_token = token_dict['token']
         transfer_id = FTS3Transfertool(external_host=external_host, token=transfer_token).submit(files=job_files, job_params=job_params, timeout=timeout)
         record_timer('core.request.submit_transfers_fts3', (time.time() - start_time) * 1000 / len(files))
     elif transfertool == 'globus':
-        logging.debug('... Starting globus xfer ...')
+        logger(logging.DEBUG, '... Starting globus xfer ...')
         job_files = []
         for file in files:
             job_file = {}
@@ -170,7 +172,7 @@ def submit_bulk_transfers(external_host, files, transfertool='fts3', job_params=
                 else:
                     job_file[key] = file[key]
             job_files.append(job_file)
-        logging.debug('job_files: %s' % job_files)
+        logger(logging.DEBUG, 'job_files: %s' % job_files)
         transfer_id = GlobusTransferTool(external_host=None).bulk_submit(submitjob=job_files, timeout=timeout)
     elif transfertool == 'mock':
         import uuid
@@ -246,15 +248,16 @@ def set_transfers_state(transfers, submitted_at, session=None):
                 raise RucioException("Failed to set requests %s tansfer %s: request doesn't exist or is not in SUBMITTING state" % (request_id, transfers[request_id]))
 
             request_type = transfers[request_id].get('request_type', None)
+
             msg = {'request-id': request_id,
-                   'request-type': str(request_type).lower() if request_type else request_type,
+                   'request-type': request_type,
                    'scope': transfers[request_id]['scope'].external,
                    'name': transfers[request_id]['name'],
                    'src-rse-id': transfers[request_id]['metadata'].get('src_rse_id', None),
                    'src-rse': transfers[request_id]['metadata'].get('src_rse', None),
                    'dst-rse-id': transfers[request_id]['metadata'].get('dst_rse_id', None),
                    'dst-rse': transfers[request_id]['metadata'].get('dst_rse', None),
-                   'state': str(transfers[request_id]['state']),
+                   'state': transfers[request_id]['state'],
                    'activity': transfers[request_id]['metadata'].get('activity', None),
                    'file-size': transfers[request_id]['metadata'].get('filesize', None),
                    'bytes': transfers[request_id]['metadata'].get('filesize', None),
@@ -267,22 +270,24 @@ def set_transfers_state(transfers, submitted_at, session=None):
                 msg['vo'] = transfers[request_id]['scope'].vo
 
             if msg['request-type']:
-                transfer_status = '%s-%s' % (msg['request-type'], msg['state'])
+                transfer_status = '%s-%s' % (msg['request-type'].name, msg['state'].name)
             else:
                 transfer_status = 'transfer-%s' % msg['state']
             transfer_status = transfer_status.lower()
+
             message_core.add_message(transfer_status, msg, session=session)
 
     except IntegrityError as error:
         raise RucioException(error.args)
 
 
-def bulk_query_transfers(request_host, transfer_ids, transfertool='fts3', timeout=None):
+def bulk_query_transfers(request_host, transfer_ids, transfertool='fts3', timeout=None, logger=logging.log):
     """
     Query the status of a transfer.
     :param request_host:  Name of the external host.
     :param transfer_ids:  List of (External-ID as a 32 character hex string)
     :param transfertool:  Transfertool name as a string.
+    :param logger:        Optional decorated logger that can be passed from the calling daemons or servers.
     :returns:             Request status information as a dictionary.
     """
 
@@ -301,17 +306,17 @@ def bulk_query_transfers(request_host, transfer_ids, transfertool='fts3', timeou
                 fts_resps[transfer_id] = Exception("Transfer id %s is not returned" % transfer_id)
             if fts_resps[transfer_id] and not isinstance(fts_resps[transfer_id], Exception):
                 for request_id in fts_resps[transfer_id]:
-                    if fts_resps[transfer_id][request_id]['file_state'] in (str(FTSState.FAILED),
-                                                                            str(FTSState.FINISHEDDIRTY),
-                                                                            str(FTSState.CANCELED)):
+                    if fts_resps[transfer_id][request_id]['file_state'] in (str(FTSState.FAILED.name),
+                                                                            str(FTSState.FINISHEDDIRTY.name),
+                                                                            str(FTSState.CANCELED.name)):
                         fts_resps[transfer_id][request_id]['new_state'] = RequestState.FAILED
-                    elif fts_resps[transfer_id][request_id]['file_state'] in str(FTSState.FINISHED):
+                    elif fts_resps[transfer_id][request_id]['file_state'] in str(FTSState.FINISHED.name):
                         fts_resps[transfer_id][request_id]['new_state'] = RequestState.DONE
         return fts_resps
     elif transfertool == 'globus':
         try:
             start_time = time.time()
-            logging.debug('transfer_ids: %s' % transfer_ids)
+            logger(logging.DEBUG, 'transfer_ids: %s' % transfer_ids)
             responses = GlobusTransferTool(external_host=None).bulk_query(transfer_ids=transfer_ids, timeout=timeout)
             record_timer('core.request.bulk_query_transfers', (time.time() - start_time) * 1000 / len(transfer_ids))
         except Exception:
@@ -352,12 +357,13 @@ def set_transfer_update_time(external_host, transfer_id, update_time=datetime.da
         raise UnsupportedOperation("Transfer %s doesn't exist or its status is not submitted." % (transfer_id))
 
 
-def query_latest(external_host, state, last_nhours=1):
+def query_latest(external_host, state, last_nhours=1, logger=logging.log):
     """
     Query the latest transfers in last n hours with state.
     :param external_host:  FTS host name as a string.
     :param state:          FTS job state as a string or a dictionary.
     :param last_nhours:    Latest n hours as an integer.
+    :param logger:         Optional decorated logger that can be passed from the calling daemons or servers.
     :returns:              Requests status information as a dictionary.
     """
 
@@ -378,10 +384,10 @@ def query_latest(external_host, state, last_nhours=1):
         if 'request_id' not in resp['job_metadata']:
             # submitted by new submitter
             try:
-                logging.debug("Transfer %s on %s is %s, decrease its updated_at." % (resp['job_id'], external_host, resp['job_state']))
+                logger(logging.DEBUG, "Transfer %s on %s is %s, decrease its updated_at." % (resp['job_id'], external_host, resp['job_state']))
                 set_transfer_update_time(external_host, resp['job_id'], datetime.datetime.utcnow() - datetime.timedelta(hours=24))
             except Exception as error:
-                logging.debug("Exception happened when updating transfer updatetime: %s" % str(error).replace('\n', ''))
+                logger(logging.DEBUG, "Exception happened when updating transfer updatetime: %s" % str(error).replace('\n', ''))
 
     return ret_resps
 
@@ -409,26 +415,23 @@ def touch_transfer(external_host, transfer_id, session=None):
 
 
 @transactional_session
-def update_transfer_state(external_host, transfer_id, state, logging_prepend_str=None, session=None):
+def update_transfer_state(external_host, transfer_id, state, session=None, logger=logging.log):
     """
     Used by poller to update the internal state of transfer,
     after the response by the external transfertool.
     :param request_host:          Name of the external host.
     :param transfer_id:           External transfer job id as a string.
     :param state:                 Request state as a string.
-    :param logging_prepend_str:   String to prepend to the logging
     :param session:               The database session to use.
+    :param logger:                Optional decorated logger that can be passed from the calling daemons or servers.
     :returns commit_or_rollback:  Boolean.
     """
 
-    prepend_str = ''
-    if logging_prepend_str:
-        prepend_str = logging_prepend_str
     try:
         if state == RequestState.LOST:
             reqs = request_core.get_requests_by_transfer(external_host, transfer_id, session=session)
             for req in reqs:
-                logging.info(prepend_str + 'REQUEST %s OF TRANSFER %s ON %s STATE %s' % (str(req['request_id']), external_host, transfer_id, str(state)))
+                logger(logging.INFO, 'REQUEST %s OF TRANSFER %s ON %s STATE %s' % (str(req['request_id']), external_host, transfer_id, str(state)))
                 src_rse_id = req.get('source_rse_id', None)
                 dst_rse_id = req.get('dest_rse_id', None)
                 src_rse = None
@@ -476,7 +479,7 @@ def update_transfer_state(external_host, transfer_id, state, logging_prepend_str
             __set_transfer_state(external_host, transfer_id, state, session=session)
         return True
     except UnsupportedOperation as error:
-        logging.warning(prepend_str + "Transfer %s on %s doesn't exist - Error: %s" % (transfer_id, external_host, str(error).replace('\n', '')))
+        logger(logging.WARNING, "Transfer %s on %s doesn't exist - Error: %s" % (transfer_id, external_host, str(error).replace('\n', '')))
         return False
 
 
@@ -627,7 +630,7 @@ def get_dsn(scope, name, dsn):
 
 @transactional_session
 def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0, limit=None, activity=None, older_than=None, rses=None, schemes=None,
-                                              bring_online=43200, retry_other_fts=False, failover_schemes=None, session=None):
+                                              bring_online=43200, retry_other_fts=False, failover_schemes=None, session=None, logger=logging.log):
     """
     Get transfer requests and the associated source replicas
     :param total_workers:         Number of total workers.
@@ -640,6 +643,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
     :param bring_online:          Bring online timeout.
     :parm retry_other_fts:        Retry other fts servers.
     :param failover_schemes:      Failover schemes.
+    :param logger:                Optional decorated logger that can be passed from the calling daemons or servers.
     :session:                     The database session in use.
     :returns:                     transfers, reqs_no_source, reqs_scheme_mismatch, reqs_only_tape_source
     """
@@ -650,6 +654,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                                                                activity=activity,
                                                                older_than=older_than,
                                                                rses=rses,
+                                                               request_state=RequestState.QUEUED,
                                                                session=session)
 
     unavailable_read_rse_ids = __get_unavailable_rse_ids(operation='read', session=session)
@@ -694,7 +699,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
         if source_rse_id in unavailable_read_rse_ids:
             continue
         if dest_rse_id in unavailable_write_rse_ids:
-            logging.warning('RSE %s is blocked for write. Will skip the submission of new jobs', dest_rse_name)
+            logger(logging.WARNING, 'RSE %s is blocked for write. Will skip the submission of new jobs', dest_rse_name)
             continue
 
         # parse source expression
@@ -703,7 +708,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
             try:
                 parsed_rses = parse_expression(source_replica_expression, session=session)
             except InvalidRSEExpression as error:
-                logging.error("Invalid RSE exception %s: %s", source_replica_expression, str(error))
+                logger(logging.ERROR, "Invalid RSE exception %s: %s", source_replica_expression, str(error))
                 continue
             else:
                 allowed_rses = [x['id'] for x in parsed_rses]
@@ -722,18 +727,18 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                                  limit_dest_schemes=transfers.get(req_id, {}).get('schemes', None),
                                  session=session)
             if len(list_hops) > 1:
-                logging.debug('From %s to %s requires multihop: %s', source_rse_id, dest_rse_id, list_hops)
+                logger(logging.DEBUG, 'From %s to %s requires multihop: %s', source_rse_id, dest_rse_id, list_hops)
                 multihop = True
                 multi_hop_dict[req_id] = (list_hops, dict_attributes, retry_count)
         except NoDistance:
-            logging.warning("Request %s: no link from %s to %s", req_id, source_rse_name, dest_rse_name)
+            logger(logging.WARNING, "Request %s: no link from %s to %s", req_id, source_rse_name, dest_rse_name)
             if req_id in reqs_scheme_mismatch:
                 reqs_scheme_mismatch.remove(req_id)
             if req_id not in reqs_no_source:
                 reqs_no_source.append(req_id)
             continue
         except RSEProtocolNotSupported:
-            logging.warning("Request %s: no matching protocol between %s and %s", req_id, source_rse_name, dest_rse_name)
+            logger(logging.WARNING, "Request %s: no matching protocol between %s and %s", req_id, source_rse_name, dest_rse_name)
             if req_id in reqs_no_source:
                 reqs_no_source.remove(req_id)
             if req_id not in reqs_scheme_mismatch:
@@ -866,10 +871,10 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                 dest_globus_endpoint_id = rse_attrs[dest_rse_id].get('globus_endpoint_id', None)
 
                 if TRANSFER_TOOL == 'fts3' and not fts_hosts:
-                    logging.error('Destination RSE %s FTS attribute not defined - SKIP REQUEST %s', dest_rse_name, req_id)
+                    logger(logging.ERROR, 'Destination RSE %s FTS attribute not defined - SKIP REQUEST %s', dest_rse_name, req_id)
                     continue
                 if TRANSFER_TOOL == 'globus' and (not dest_globus_endpoint_id or not source_globus_endpoint_id):
-                    logging.error('Destination RSE %s Globus endpoint attributes not defined - SKIP REQUEST %s', dest_rse_name, req_id)
+                    logger(logging.ERROR, 'Destination RSE %s Globus endpoint attributes not defined - SKIP REQUEST %s', dest_rse_name, req_id)
                     continue
                 if retry_count is None:
                     retry_count = 0
@@ -900,7 +905,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                 common_checksum_names = set(source_rse_checksums).intersection(dest_rse_checksums)
 
                 if len(common_checksum_names) == 0:
-                    logging.info('No common checksum method. Verifying destination only.')
+                    logger(logging.INFO, 'No common checksum method. Verifying destination only.')
                     verify_checksum = 'destination'
 
                 # VI - Fill the transfer dictionary including file_metadata
@@ -908,7 +913,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                                  'scope': scope,
                                  'name': name,
                                  'activity': activity,
-                                 'request_type': str(RequestType.TRANSFER).lower(),
+                                 'request_type': RequestType.TRANSFER,
                                  'src_type': transfer_src_type,
                                  'dst_type': transfer_dst_type,
                                  'src_rse': rse,
@@ -951,9 +956,9 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                 if archive_timeout and (rses_info[dest_rse_id]['rse_type'] == RSEType.TAPE or rses_info[dest_rse_id]['rse_type'] == 'TAPE'):
                     try:
                         transfers[req_id]['archive_timeout'] = int(archive_timeout)
-                        logging.debug('Added archive timeout to transfer.')
+                        logger(logging.DEBUG, 'Added archive timeout to transfer.')
                     except ValueError:
-                        logging.warning('Could not set archive_timeout for %s. Must be integer.', dest_url)
+                        logger(logging.WARNING, 'Could not set archive_timeout for %s. Must be integer.', dest_url)
                         pass
             else:
                 # parse allow tape source expression, not finally version.
@@ -1072,7 +1077,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     transfers[req_id]['use_ipv4'] = True
 
         except Exception:
-            logging.critical("Exception happened when trying to get transfer for request %s: %s" % (req_id, traceback.format_exc()))
+            logger(logging.CRITICAL, "Exception happened when trying to get transfer for request %s: %s" % (req_id), exc_info=True)
             break
 
     # checking OIDC AuthN/Z support per destination and soucre RSEs;
@@ -1139,7 +1144,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                                      dataset_meta=None,
                                      session=session)
                     except Exception as error:
-                        logging.error('Problem adding replicas %s:%s on %s : %s', scope, name, dest_rse_name, str(error))
+                        logger(logging.ERROR, 'Problem adding replicas %s:%s on %s : %s', scope, name, dest_rse_name, str(error))
 
                     req_attributes = {'activity': transfers[req_id]['file_metadata']['activity'],
                                       'source_replica_expression': None,
@@ -1163,12 +1168,12 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     # If a request already exists, new_req will be an empty list.
                     if not new_req:
                         # Need to fail all the intermediate requests + the initial one and exit the multihop loop
-                        logging.warning('Multihop : A request already exists for the transfer between %s and %s. Will cancel all the parent requests', source_rse_name, dest_rse_name)
+                        logger(logging.WARNING, 'Multihop : A request already exists for the transfer between %s and %s. Will cancel all the parent requests', source_rse_name, dest_rse_name)
                         parent_requests.append(req_id)
                         try:
                             set_requests_state(request_ids=parent_requests, new_state=RequestState.FAILED, session=session)
                         except UnsupportedOperation:
-                            logging.error('Multihop : Cannot cancel all the parent requests : %s', str(parent_requests))
+                            logger(logging.ERROR, 'Multihop : Cannot cancel all the parent requests : %s', str(parent_requests))
 
                         # Remove from the transfer dictionary all the requests
                         for cur_req_id in parent_requests:
@@ -1177,7 +1182,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     new_req_id = new_req[0]['id']
                     parent_requests.append(new_req_id)
                     set_requests_state(request_ids=[new_req_id, ], new_state=RequestState.QUEUED, session=session)
-                    logging.debug('New request created for the transfer between %s and %s : %s', source_rse_name, dest_rse_name, new_req_id)
+                    logger(logging.DEBUG, 'New request created for the transfer between %s and %s : %s', source_rse_name, dest_rse_name, new_req_id)
 
                     # I - Here we will compute the destination URL
                     # I.1 - Get destination protocol
@@ -1230,7 +1235,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                                      'scope': scope,
                                      'name': name,
                                      'activity': transfers[req_id]['file_metadata']['activity'],
-                                     'request_type': str(RequestType.TRANSFER).lower(),
+                                     'request_type': RequestType.TRANSFER,
                                      'src_type': transfer_src_type,
                                      'dst_type': transfer_dst_type,
                                      'src_rse': source_rse_name,
@@ -1283,8 +1288,8 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
 
 
 @read_session
-def __list_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
-                                                 limit=None, activity=None, older_than=None, rses=None, session=None):
+def __list_transfer_requests_and_source_replicas(total_workers=0, worker_number=0, limit=None, activity=None,
+                                                 older_than=None, rses=None, request_state=None, session=None) -> "List[Tuple]":
     """
     List requests with source replicas
     :param total_workers:     Number of total workers.
@@ -1296,6 +1301,10 @@ def __list_transfer_requests_and_source_replicas(total_workers=0, worker_number=
     :param session:          Database session to use.
     :returns:                List.
     """
+
+    if request_state is None:
+        request_state = RequestState.QUEUED
+
     sub_requests = session.query(models.Request.id,
                                  models.Request.rule_id,
                                  models.Request.scope,
@@ -1308,12 +1317,12 @@ def __list_transfer_requests_and_source_replicas(total_workers=0, worker_number=
                                  models.Request.previous_attempt_id,
                                  models.Request.dest_rse_id,
                                  models.Request.retry_count,
-                                 models.Request.account)\
-        .with_hint(models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_UPD_IDX)", 'oracle')\
-        .filter(models.Request.state == RequestState.QUEUED)\
-        .filter(models.Request.request_type == RequestType.TRANSFER)\
-        .join(models.RSE, models.RSE.id == models.Request.dest_rse_id)\
-        .filter(models.RSE.deleted == false())\
+                                 models.Request.account) \
+        .with_hint(models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_UPD_IDX)", 'oracle') \
+        .filter(models.Request.state == request_state) \
+        .filter(models.Request.request_type == RequestType.TRANSFER) \
+        .join(models.RSE, models.RSE.id == models.Request.dest_rse_id) \
+        .filter(models.RSE.deleted == false()) \
         .filter(models.RSE.availability.in_((2, 3, 6, 7)))
 
     if isinstance(older_than, datetime.datetime):
@@ -1349,19 +1358,19 @@ def __list_transfer_requests_and_source_replicas(total_workers=0, worker_number=
                           sub_requests.c.retry_count,
                           models.Source.url,
                           models.Source.ranking,
-                          models.Distance.ranking)\
+                          models.Distance.ranking) \
         .outerjoin(models.RSEFileAssociation, and_(sub_requests.c.scope == models.RSEFileAssociation.scope,
                                                    sub_requests.c.name == models.RSEFileAssociation.name,
                                                    models.RSEFileAssociation.state == ReplicaState.AVAILABLE,
-                                                   sub_requests.c.dest_rse_id != models.RSEFileAssociation.rse_id))\
-        .with_hint(models.RSEFileAssociation, "+ index(replicas REPLICAS_PK)", 'oracle')\
+                                                   sub_requests.c.dest_rse_id != models.RSEFileAssociation.rse_id)) \
+        .with_hint(models.RSEFileAssociation, "+ index(replicas REPLICAS_PK)", 'oracle') \
         .outerjoin(models.RSE, and_(models.RSE.id == models.RSEFileAssociation.rse_id,
-                                    models.RSE.deleted == false()))\
+                                    models.RSE.deleted == false())) \
         .outerjoin(models.Source, and_(sub_requests.c.id == models.Source.request_id,
-                                       models.RSE.id == models.Source.rse_id))\
-        .with_hint(models.Source, "+ index(sources SOURCES_PK)", 'oracle')\
+                                       models.RSE.id == models.Source.rse_id)) \
+        .with_hint(models.Source, "+ index(sources SOURCES_PK)", 'oracle') \
         .outerjoin(models.Distance, and_(sub_requests.c.dest_rse_id == models.Distance.dest_rse_id,
-                                         models.RSEFileAssociation.rse_id == models.Distance.src_rse_id))\
+                                         models.RSEFileAssociation.rse_id == models.Distance.src_rse_id)) \
         .with_hint(models.Distance, "+ index(distances DISTANCES_PK)", 'oracle')
 
     if rses:
@@ -1396,26 +1405,27 @@ def __set_transfer_state(external_host, transfer_id, new_state, session=None):
 
 
 @read_session
-def __get_unavailable_rse_ids(operation, session=None):
+def __get_unavailable_rse_ids(operation, session=None, logger=logging.log):
     """
+    :param logger:   Optional decorated logger that can be passed from the calling daemons or servers.
     Get unavailable rse ids for a given operation : read, write, delete
     """
 
     if operation not in ['read', 'write', 'delete']:
-        logging.error("Wrong operation specified : %s" % (operation))
+        logger(logging.ERROR, "Wrong operation specified : %s" % (operation))
         return []
     key = 'unavailable_%s_rse_ids' % operation
     result = REGION_SHORT.get(key)
     if isinstance(result, NoValue):
         try:
-            logging.debug("Refresh unavailable %s rses" % operation)
+            logger(logging.DEBUG, "Refresh unavailable %s rses" % operation)
             availability_key = 'availability_%s' % operation
             unavailable_rses = list_rses(filters={availability_key: False}, session=session)
             unavailable_rse_ids = [rse['id'] for rse in unavailable_rses]
             REGION_SHORT.set(key, unavailable_rse_ids)
             return unavailable_rse_ids
         except Exception:
-            logging.warning("Failed to refresh unavailable %s rses, error: %s" % (operation, traceback.format_exc()))
+            logger(logging.ERROR, "Failed to refresh unavailable %s rses, error" % (operation), exc_info=True)
             return []
     return result
 
