@@ -1,4 +1,5 @@
-# Copyright 2016-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2016-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,14 +16,15 @@
 # Authors:
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2016-2018
 # - Vincent Garonne <vgaronne@gmail.com>, 2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019-2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
-# - Patrick Austin, <patrick.austin@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
+# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 import datetime
 import logging
@@ -31,26 +33,20 @@ import random
 import socket
 import threading
 import time
-
-from sys import exc_info, stdout
+from sys import exc_info
 from traceback import format_exception
 
-from rucio.db.sqla.constants import LifetimeExceptionsState
-from rucio.common.config import config_get
-from rucio.common.exception import InvalidRSEExpression, RuleNotFound
-from rucio.core import heartbeat
 import rucio.core.lifetime_exception
+import rucio.db.sqla.util
+from rucio.common import exception
+from rucio.common.exception import InvalidRSEExpression, RuleNotFound
+from rucio.common.logging import setup_logging
+from rucio.core import heartbeat
 from rucio.core.lock import get_dataset_locks
 from rucio.core.rse import get_rse_name, get_rse_vo
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import get_rules_beyond_eol, update_rule
-
-logging.basicConfig(stream=stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
+from rucio.db.sqla.constants import LifetimeExceptionsState
 
 GRACEFUL_STOP = threading.Event()
 
@@ -203,6 +199,11 @@ def run(threads=1, bulk=100, date_check=None, dry_run=True, grace_period=86400,
     """
     Starts up the atropos threads.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise exception.DatabaseException('Database was not updated, daemon won\'t start')
+
     if not date_check:
         date_check = datetime.datetime.now()
     else:
