@@ -1,4 +1,5 @@
-# Copyright 2015-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2015-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,15 +14,13 @@
 # limitations under the License.
 #
 # Authors:
-# - Fernando Lopez <fernando.e.lopez@gmail.com>, 2015-2016
-# - Martin Barisits <martin.barisits@cern.ch>, 2017
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2017
-# - Vincent Garonne <vgaronne@gmail.com>, 2018
+# - Fernando López <felopez@cern.ch>, 2015-2016
+# - Martin Barisits <martin.barisits@cern.ch>, 2017-2019
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2017-2020
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2018
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-#
-# PY3K COMPATIBLE
 
 from __future__ import division
 
@@ -58,7 +57,7 @@ def consistency(rse, delta, configuration, cache_dir, results_dir):
     results_path = os.path.join(results_dir, '{0}_{1}'.format(rse, rsedate.strftime('%Y%m%d')))  # pylint: disable=no-member
 
     if os.path.exists(results_path + '.bz2') or os.path.exists(results_path):
-        logger.warn('Consistency check for "%s" (dump dated %s) already done, skipping check', rse, rsedate.strftime('%Y%m%d'))  # pylint: disable=no-member
+        logger.warning('Consistency check for "%s" (dump dated %s) already done, skipping check', rse, rsedate.strftime('%Y%m%d'))  # pylint: disable=no-member
         return None
 
     rrdump_prev = ReplicaFromHDFS.download(rse, rsedate - delta, cache_dir=cache_dir)
@@ -121,7 +120,7 @@ def bz2_compress_file(source, chunk_size=65000):
             chunk = plain.read(chunk_size)
             if not chunk:
                 break
-            compressed.write(chunk)
+            compressed.write(chunk.encode())
     os.remove(source)
     return destination
 
@@ -230,7 +229,6 @@ def check(queue, retry, terminate, logpipe, cache_dir, results_dir, keep_dumps,
             rse, attemps = queue.get(timeout=30)
         except Queue.Empty:
             continue
-
         start = datetime.now()
         try:
             logger.debug('Checking "%s"', rse)
@@ -240,6 +238,7 @@ def check(queue, retry, terminate, logpipe, cache_dir, results_dir, keep_dumps,
                 process_output(output)
         except:
             success = False
+            class_, desc = sys.exc_info()[0:2]
         else:
             success = True
         finally:
@@ -247,7 +246,6 @@ def check(queue, retry, terminate, logpipe, cache_dir, results_dir, keep_dumps,
             if success:
                 logger.info('SUCCESS checking "%s" in %d minutes', rse, elapsed)
             else:
-                class_, desc = sys.exc_info()[0:2]
                 logger.error('Check of "%s" failed in %d minutes, %d remaining attemps: (%s: %s)', rse, elapsed, attemps, class_.__name__, desc)
 
         if not keep_dumps:
