@@ -1,4 +1,5 @@
-# Copyright 2014-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2014-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,14 +14,15 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2020
-# - Wen Guan <wguan.icedew@gmail.com>, 2014-2015
-# - Vincent Garonne <vgaronne@gmail.com>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2017
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2021
+# - Wen Guan <wen.guan@cern.ch>, 2014-2015
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2016-2019
 # - Robert Illingworth <illingwo@fnal.gov>, 2018
-#
-# PY3K COMPATIBLE
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+# - Eric Vaandering <ewv@fnal.gov>, 2019-2021
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 '''
    Hermes is a daemon to deliver messages: to a messagebroker via STOMP, or emails via SMTP.
@@ -32,34 +34,27 @@ import os
 import random
 import smtplib
 import socket
-import sys
 import threading
 import time
 import traceback
-
 from email.mime.text import MIMEText
-from six import PY2
-from prometheus_client import Counter
-from sqlalchemy.orm.exc import NoResultFound
 
 import stomp
+from prometheus_client import Counter
+from six import PY2
+from sqlalchemy.orm.exc import NoResultFound
 
+import rucio.db.sqla.util
+from rucio.common import exception
 from rucio.common.config import config_get, config_get_int, config_get_bool
+from rucio.common.logging import setup_logging
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.message import retrieve_messages, delete_messages
 from rucio.core.monitor import record_counter
 
-
 logging.getLogger('requests').setLevel(logging.CRITICAL)
 logging.getLogger('stomp').setLevel(logging.CRITICAL)
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 GRACEFUL_STOP = threading.Event()
 
 RECONNECT_COUNTER = Counter('rucio_daemons_hermes_reconnect', 'Counts Hermes reconnects to different ActiveMQ brokers', labelnames=('host',))
@@ -110,7 +105,10 @@ def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=10):
 
                 msg['From'] = email_from
                 msg['To'] = ', '.join(message['payload']['to'])
-                msg['Subject'] = message['payload']['subject'].encode('utf-8')
+                if PY2:
+                    msg['Subject'] = message['payload']['subject'].encode('utf-8')
+                else:
+                    msg['Subject'] = message['payload']['subject']
 
                 if send_email:
                     smtp = smtplib.SMTP()
@@ -286,8 +284,8 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
                                           'payload': json.dumps(message['payload']),
                                           'event_type': message['event_type']})
                     except ValueError:
-                        logging.warn('Cannot serialize payload to JSON: %s',
-                                     str(message['payload']))
+                        logging.warning('Cannot serialize payload to JSON: %s',
+                                        str(message['payload']))
                         to_delete.append({'id': message['id'],
                                           'created_at': message['created_at'],
                                           'updated_at': message['created_at'],
@@ -295,15 +293,15 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
                                           'event_type': message['event_type']})
                         continue
                     except stomp.exception.NotConnectedException as error:
-                        logging.warn('Could not deliver message due to NotConnectedException: %s',
-                                     str(error))
+                        logging.warning('Could not deliver message due to NotConnectedException: %s',
+                                        str(error))
                         continue
                     except stomp.exception.ConnectFailedException as error:
-                        logging.warn('Could not deliver message due to ConnectFailedException: %s',
-                                     str(error))
+                        logging.warning('Could not deliver message due to ConnectFailedException: %s',
+                                        str(error))
                         continue
                     except Exception as error:
-                        logging.warn('Could not deliver message: %s', str(error))
+                        logging.warning('Could not deliver message: %s', str(error))
                         logging.critical(traceback.format_exc())
                         continue
 
@@ -396,6 +394,10 @@ def run(once=False, send_email=True, threads=1, bulk=1000, delay=10, broker_time
     '''
     Starts up the hermes threads.
     '''
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise exception.DatabaseException('Database was not updated, daemon won\'t start')
 
     logging.info('resolving brokers')
 
