@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-# Copyright 2014-2020 CERN
+# -*- coding: utf-8 -*-
+# Copyright 2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,16 +14,11 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2020
-# - Wen Guan <wguan.icedew@gmail.com>, 2014-2015
-# - Vincent Garonne <vgaronne@gmail.com>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2017
-# - Robert Illingworth <illingwo@fnal.gov>, 2018
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2020
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2020
+# - Eric Vaandering <ewv@fnal.gov>, 2021
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
 '''
    Hermes2 is a daemon that get the messages and sends them to external services (influxDB, ES, ActiveMQ).
@@ -43,31 +38,24 @@ import sys
 import threading
 import time
 import traceback
-
 from email.mime.text import MIMEText
-from six import PY2
-from prometheus_client import Counter
 
 import requests
 import stomp
+from prometheus_client import Counter
+from six import PY2
 
+import rucio.db.sqla.util
 from rucio.common.config import config_get, config_get_int, config_get_bool
+from rucio.common.exception import ConfigNotFound, DatabaseException
+from rucio.common.logging import setup_logging
 from rucio.core import heartbeat
 from rucio.core.config import get
 from rucio.core.message import retrieve_messages, delete_messages, update_messages_services
 from rucio.core.monitor import record_counter
-from rucio.common.exception import ConfigNotFound
-
 
 logging.getLogger('requests').setLevel(logging.CRITICAL)
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 GRACEFUL_STOP = threading.Event()
 
 RECONNECT_COUNTER = Counter('rucio_daemons_hermes2_reconnect', 'Counts Hermes2 reconnects to different ActiveMQ brokers', labelnames=('host',))
@@ -212,14 +200,14 @@ def deliver_to_activemq(messages, conns, destination, username, password, use_ss
             to_delete.append(message['id'])
             continue
         except stomp.exception.NotConnectedException as error:
-            logging.warn('%s [broker] Could not deliver message due to NotConnectedException: %s',
-                         prepend_str,
-                         str(error))
+            logging.warning('%s [broker] Could not deliver message due to NotConnectedException: %s',
+                            prepend_str,
+                            str(error))
             continue
         except stomp.exception.ConnectFailedException as error:
-            logging.warn('%s [broker] Could not deliver message due to ConnectFailedException: %s',
-                         prepend_str,
-                         str(error))
+            logging.warning('%s [broker] Could not deliver message due to ConnectFailedException: %s',
+                            prepend_str,
+                            str(error))
             continue
         except Exception as error:
             logging.error('%s [broker] Could not deliver message: %s',
@@ -287,7 +275,10 @@ def deliver_emails(messages, prepend_str):
 
             msg['From'] = email_from
             msg['To'] = ', '.join(message['payload']['to'])
-            msg['Subject'] = message['payload']['subject'].encode('utf-8')
+            if PY2:
+                msg['Subject'] = message['payload']['subject'].encode('utf-8')
+            else:
+                msg['Subject'] = message['payload']['subject']
 
             try:
                 smtp = smtplib.SMTP()
@@ -362,8 +353,8 @@ def aggregate_to_influx(messages, bin_size, endpoint, prepend_str):
         elif event_type in ['deletion-failed', 'deletion-done']:
             created_at = message['created_at']
             if bin_size == '1m':
-                created_at = created_at.replace(second=0, microsecond=0)
-            created_at = int(created_at.strftime('%s')) * 1000000000
+                created_at = created_at.replace(second=0, microsecond=0, tzinfo=datetime.timezone.utc).timestamp()
+            created_at = int(created_at) * 1000000000
             created_at += microsecond
             if created_at not in bins:
                 bins[created_at] = {}
@@ -571,6 +562,10 @@ def run(once=False, threads=1, bulk=1000, sleep_time=10, broker_timeout=3):
     '''
     Starts up the hermes2 threads.
     '''
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise DatabaseException('Database was not updated, daemon won\'t start')
 
     logging.info('starting hermes2 threads')
     thread_list = [threading.Thread(target=hermes2, kwargs={'thread': cnt,
