@@ -1,4 +1,5 @@
-# Copyright 2015-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2015-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,11 +18,9 @@
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 # - Eric Vaandering <ewv@fnal.gov>, 2020
-#
-# PY3K COMPATIBLE
 
 """
 Judge-Injector is a daemon to asynchronously create replication rules
@@ -30,34 +29,25 @@ Judge-Injector is a daemon to asynchronously create replication rules
 import logging
 import os
 import socket
-import sys
 import threading
 import time
-import traceback
-
 from copy import deepcopy
 from datetime import datetime, timedelta
-from re import match
 from random import randint
+from re import match
 
 from sqlalchemy.exc import DatabaseError
 
-from rucio.common.config import config_get
+import rucio.db.sqla.util
+from rucio.common.logging import formatted_logger, setup_logging
 from rucio.common.exception import (DatabaseException, RuleNotFound, RSEBlacklisted, RSEWriteBlocked,
                                     ReplicationRuleCreationTemporaryFailed, InsufficientAccountLimit)
 from rucio.core.heartbeat import live, die, sanity_check
-from rucio.core.rule import inject_rule, get_injected_rules, update_rule
 from rucio.core.monitor import record_counter
+from rucio.core.rule import inject_rule, get_injected_rules, update_rule
 
 graceful_stop = threading.Event()
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 
 def rule_injector(once=False):
     """
@@ -72,13 +62,17 @@ def rule_injector(once=False):
 
     # Make an initial heartbeat so that all judge-inectors have the correct worker number on the next try
     executable = 'judge-injector'
-    live(executable=executable, hostname=hostname, pid=pid, thread=current_thread, older_than=2 * 60 * 60)
+    heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=current_thread, older_than=2 * 60 * 60)
+    prefix = 'judge-injector[%i/%i] ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+    logger = formatted_logger(logging.log, prefix + '%s')
     graceful_stop.wait(1)
 
     while not graceful_stop.is_set():
         try:
             # heartbeat
             heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=current_thread, older_than=2 * 60 * 60)
+            prefix = 'judge-injector[%i/%i] ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+            logger = formatted_logger(logging.log, prefix + '%s')
 
             start = time.time()
 
@@ -92,63 +86,63 @@ def rule_injector(once=False):
                                        worker_number=heartbeat['assign_thread'],
                                        limit=100,
                                        blacklisted_rules=[key for key in paused_rules])
-            logging.debug('rule_injector[%s/%s] index query time %f fetch size is %d' % (heartbeat['assign_thread'], heartbeat['nr_threads'], time.time() - start, len(rules)))
+            logger(logging.DEBUG, 'index query time %f fetch size is %d' % (time.time() - start, len(rules)))
 
             if not rules and not once:
-                logging.debug('rule_injector[%s/%s] did not get any work (paused_rules=%s)' % (heartbeat['assign_thread'], heartbeat['nr_threads'], str(len(paused_rules))))
+                logger(logging.DEBUG, 'did not get any work (paused_rules=%s)' % str(len(paused_rules)))
                 graceful_stop.wait(60)
             else:
                 for rule in rules:
                     rule_id = rule[0]
-                    logging.info('rule_injector[%s/%s]: Injecting rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
+                    logger(logging.INFO, 'Injecting rule %s' % rule_id)
                     if graceful_stop.is_set():
                         break
                     try:
                         start = time.time()
-                        inject_rule(rule_id=rule_id)
-                        logging.debug('rule_injector[%s/%s]: injection of %s took %f' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id, time.time() - start))
+                        inject_rule(rule_id=rule_id, logger=logger)
+                        logger(logging.DEBUG, 'injection of %s took %f' % (rule_id, time.time() - start))
                     except (DatabaseException, DatabaseError) as e:
                         if match('.*ORA-00054.*', str(e.args[0])):
                             paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
                             record_counter('rule.judge.exceptions.LocksDetected')
-                            logging.warning('rule_injector[%s/%s]: Locks detected for %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
+                            logger(logging.WARNING, 'Locks detected for %s' % rule_id)
                         elif match('.*QueuePool.*', str(e.args[0])):
-                            logging.warning(traceback.format_exc())
+                            logger(logging.WARNING, 'DatabaseException', exc_info=True)
                             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                         elif match('.*ORA-03135.*', str(e.args[0])):
-                            logging.warning(traceback.format_exc())
+                            logger(logging.WARNING, 'DatabaseException', exc_info=True)
                             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                         else:
-                            logging.error(traceback.format_exc())
+                            logger(logging.ERROR, 'DatabaseException', exc_info=True)
                             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except (RSEBlacklisted, RSEWriteBlocked) as e:
                         paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
-                        logging.warning('rule_injector[%s/%s]: RSEBlacklisted for rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
+                        logger(logging.WARNING, 'RSEBlacklisted for rule %s' % rule_id)
                         record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except ReplicationRuleCreationTemporaryFailed as e:
                         paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
-                        logging.warning('rule_injector[%s/%s]: ReplicationRuleCreationTemporaryFailed for rule %s' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
+                        logger(logging.WARNING, 'ReplicationRuleCreationTemporaryFailed for rule %s' % rule_id)
                         record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except RuleNotFound:
                         pass
                     except InsufficientAccountLimit:
                         # A rule with InsufficientAccountLimit on injection hangs there potentially forever
                         # It should be marked as SUSPENDED
-                        logging.info('rule_injector[%s/%s]: Marking rule %s as SUSPENDED due to InsufficientAccountLimit' % (heartbeat['assign_thread'], heartbeat['nr_threads'], rule_id))
+                        logger(logging.INFO, 'Marking rule %s as SUSPENDED due to InsufficientAccountLimit' % rule_id)
                         update_rule(rule_id=rule_id, options={'state': 'SUSPENDED'})
 
         except (DatabaseException, DatabaseError) as e:
             if match('.*QueuePool.*', str(e.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, 'DatabaseException', exc_info=True)
                 record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
             elif match('.*ORA-03135.*', str(e.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, 'DatabaseException', exc_info=True)
                 record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
             else:
-                logging.critical(traceback.format_exc())
+                logger(logging.CRITICAL, 'DatabaseException', exc_info=True)
                 record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
         except Exception as e:
-            logging.critical(traceback.format_exc())
+            logger(logging.CRITICAL, 'Exception', exc_info=True)
             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
         if once:
             break
@@ -168,6 +162,10 @@ def run(once=False, threads=1):
     """
     Starts up the Judge-Injector threads.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise DatabaseException('Database was not updated, daemon won\'t start')
 
     executable = 'judge-injector'
     hostname = socket.gethostname()
