@@ -1,4 +1,5 @@
-# Copyright 2016-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2016-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,14 +14,13 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2016-2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2016-2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2019-2021
+# - Brandon White <bjwhite@fnal.gov>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 '''
 Light Reaper is a daemon to manage temporary object/file deletion.
@@ -36,27 +36,21 @@ import threading
 import time
 import traceback
 
-from rucio.common.config import config_get, config_get_bool
+import rucio.db.sqla.util
+from rucio.common.config import config_get_bool
 from rucio.common.exception import (SourceNotFound, DatabaseException, ServiceUnavailable,
                                     RSEAccessDenied, RSENotFound, ResourceTemporaryUnavailable, VONotFound)
+from rucio.common.logging import setup_logging
 from rucio.core import rse as rse_core
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.message import add_message
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.temporary_did import (list_expired_temporary_dids, delete_temporary_dids)
-from rucio.rse import rsemanager as rsemgr
 from rucio.core.vo import list_vos
-
+from rucio.rse import rsemanager as rsemgr
 
 logging.getLogger("requests").setLevel(logging.CRITICAL)
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 GRACEFUL_STOP = threading.Event()
 
 
@@ -181,7 +175,7 @@ def stop(signum=None, frame=None):
     GRACEFUL_STOP.set()
 
 
-def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_rses=False,
+def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None,
         exclude_rses=None, include_rses=None, vos=None, delay_seconds=0):
     """
     Starts up the reaper threads.
@@ -196,6 +190,11 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_r
     :param vos: VOs on which to look for RSEs. Only used in multi-VO mode.
                 If None, we either use all VOs if run from "def", or the current VO otherwise.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise DatabaseException('Database was not updated, daemon won\'t start')
+
     logging.info('main: starting processes')
 
     multi_vo = config_get_bool('common', 'multi_vo', raise_exception=False, default=False)
@@ -203,9 +202,6 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_r
         if vos:
             logging.warning('Ignoring argument vos, this is only applicable in a multi-VO setup.')
         vos = ['def']
-
-        # Preserve the old behaviour until a feature release
-        all_rses = rse_core.list_rses()
     else:
         if vos:
             invalid = set(vos) - set([v['vo'] for v in list_vos()])
@@ -216,37 +212,31 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None, all_r
             vos = [v['vo'] for v in list_vos()]
         logging.info('Light Reaper: This instance will work on VO%s: %s' % ('s' if len(vos) > 1 else '', ', '.join([v for v in vos])))
 
-        # Only use the include/exclude RSE options for m-VO until a feature release
-        all_rses = []
-        for vo in vos:
-            vo_rses = rse_core.list_rses(filters={'vo': vo})
-            if vo_rses:
-                all_rses.extend(vo_rses)
+    all_rses = []
+    for vo in vos:
+        all_rses.extend(rse_core.list_rses(filters={'vo': vo}))
 
-    if all_rses:
-        rses = all_rses
+    if rses:
+        invalid = set(rses) - set([rse['rse'] for rse in all_rses])
+        if invalid:
+            msg = 'RSE{} {} cannot be found'.format('s' if len(invalid) > 1 else '',
+                                                    ', '.join([repr(rse) for rse in invalid]))
+            raise RSENotFound(msg)
+        rses = [rse for rse in all_rses if rse['rse'] in rses]
     else:
-        if rses:
-            invalid = set(rses) - set([rse['rse'] for rse in all_rses])
-            if invalid:
-                msg = 'RSE{} {} cannot be found'.format('s' if len(invalid) > 1 else '',
-                                                        ', '.join([repr(rse) for rse in invalid]))
-                raise RSENotFound(msg)
-            rses = [rse for rse in all_rses if rse['rse'] in rses]
-        else:
-            rses = all_rses
+        rses = all_rses
 
-        if exclude_rses:
-            excluded_rses = parse_expression(exclude_rses)
-            rses = [rse for rse in rses if rse not in excluded_rses]
+    if exclude_rses:
+        excluded_rses = parse_expression(exclude_rses)
+        rses = [rse for rse in rses if rse not in excluded_rses]
 
-        if include_rses:
-            included_rses = parse_expression(include_rses)
-            rses = [rse for rse in rses if rse in included_rses]
+    if include_rses:
+        included_rses = parse_expression(include_rses)
+        rses = [rse for rse in rses if rse in included_rses]
 
-        if not rses:
-            logging.error('Light Reaper: No RSEs found. Exiting.')
-            return
+    if not rses:
+        logging.error('Light Reaper: No RSEs found. Exiting.')
+        return
 
     threads = []
     for worker in range(total_workers):
