@@ -1,4 +1,5 @@
-# Copyright 2017-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2017-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,9 +16,10 @@
 # Authors:
 # - Vitjan Zavrtanik <vitjan.zavrtanik@cern.ch>, 2017
 # - Vincent Garonne <vgaronne@gmail.com>, 2017-2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Eric Vaandering <ewv@fnal.gov>, 2020
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
 """
 Daemon for distributing sonar test files to available RSE's
@@ -27,22 +29,17 @@ import glob
 import logging
 import os
 import subprocess
-import sys
 import threading
 import time
 
+import rucio.db.sqla.util
 from rucio.client.client import Client
+from rucio.common import exception
 from rucio.common.config import config_get
-from rucio.common.exception import (DuplicateRule, InsufficientAccountLimit, ReplicationRuleCreationTemporaryFailed,
-                                    RSEBlacklisted, RSEWriteBlocked)
+from rucio.common.exception import DuplicateRule, InsufficientAccountLimit, RSEBlacklisted, RSEWriteBlocked, ReplicationRuleCreationTemporaryFailed
+from rucio.common.logging import setup_logging
 
 GRACEFUL_STOP = threading.Event()
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
 
 
 def rename_files(tdir, pattern, new_name):
@@ -133,6 +130,11 @@ def run():
     """
     Runs the distribution daemon
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise exception.DatabaseException('Database was not updated, daemon won\'t start')
+
     thread = threading.Thread(target=run_distribution, kwargs={})
     thread.start()
     while thread and thread.isAlive():
