@@ -1,4 +1,5 @@
-# Copyright 2017-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2017-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +16,11 @@
 # Authors:
 # - Vitjan Zavrtanik <vitjan.zavrtanik@cern.ch>, 2017
 # - Vincent Garonne <vgaronne@gmail.com>, 2017-2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Eric Vaandering <ewv@fnal.gov>, 2020
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
 """
 Daemon for sending Sonar tests to available RSE's.
@@ -29,24 +31,19 @@ from __future__ import division
 import itertools
 import logging
 import random
-import sys
 import threading
 import time
 
 from requests import ConnectionError
+
+import rucio.db.sqla.util
 from rucio.client.client import Client
 from rucio.common.config import config_get
-from rucio.common.exception import (AccessDenied, DuplicateRule, RSEBlacklisted, RSEWriteBlocked,
+from rucio.common.exception import (AccessDenied, DatabaseException, DuplicateRule, RSEBlacklisted, RSEWriteBlocked,
                                     ReplicationRuleCreationTemporaryFailed, RuleNotFound)
+from rucio.common.logging import setup_logging
 from rucio.daemons.sonar.sonar.get_current_traffic import get_link_traffic
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 
 GRACEFUL_STOP = threading.Event()
 DATASET_PREFIX = config_get('sonar', 'dataset_prefix')
@@ -417,6 +414,11 @@ def run():
     """
     Starts the Sonar thread.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise DatabaseException('Database was not updated, daemon won\'t start')
+
     threads = []
     threads.append(threading.Thread(target=sonar_tests, kwargs={}, name='Sonar_test_v3'))
     for thread in threads:
