@@ -1,4 +1,5 @@
-# Copyright 2018-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2018-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,23 +14,19 @@
 # limitations under the License.
 #
 # Authors:
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
-# - Vincent Garonne <vgaronne@gmail.com>, 2014-2018
-# - David Cameron <d.g.cameron@gmail.com>, 2014
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2015-2018
-# - Wen Guan <wen.guan@cern.ch>, 2015
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2017
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018-2020
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Robert Illingworth <illingwo@fnal.gov>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 # - Eric Vaandering <ewv@fnal.gov>, 2020
-#
-# PY3K COMPATIBLE
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2021
 
 import logging
 import os
@@ -37,21 +34,19 @@ import re
 import socket
 import threading
 import time
-
 from datetime import datetime
 from json import loads
 from math import exp
-from sys import exc_info, stdout
+from sys import exc_info
 from traceback import format_exception
 
-
-from rucio.db.sqla.constants import DIDType, SubscriptionState
+import rucio.db.sqla.util
 from rucio.common.exception import (DatabaseException, DataIdentifierNotFound, InvalidReplicationRule, DuplicateRule,
                                     RSEBlacklisted, RSEWriteBlocked, InvalidRSEExpression, InsufficientTargetRSEs,
                                     InsufficientAccountLimit, InputValidationError, RSEOverQuota,
                                     ReplicationRuleCreationTemporaryFailed, InvalidRuleWeight,
                                     StagingAreaRuleRequiresLifetime, SubscriptionWrongParameter, SubscriptionNotFound)
-from rucio.common.config import config_get
+from rucio.common.logging import setup_logging
 from rucio.common.schema import validate_schema
 from rucio.common.utils import chunks
 from rucio.core import monitor, heartbeat
@@ -61,15 +56,7 @@ from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rse_selector import RSESelector
 from rucio.core.rule import add_rule, list_rules, get_rule
 from rucio.core.subscription import list_subscriptions, update_subscription
-
-
-logging.basicConfig(stream=stdout,
-                    level=getattr(logging,
-                                  config_get('common', 'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
+from rucio.db.sqla.constants import DIDType, SubscriptionState
 
 graceful_stop = threading.Event()
 
@@ -145,6 +132,16 @@ def is_matching_subscription(subscription, did, metadata):
                     break
             if not match_account:
                 return False
+        elif key == 'did_type':
+            match_did_type = False
+            if not isinstance(values, list):
+                values = [values]
+            for did_type in values:
+                if did_type == metadata['did_type'].name:
+                    match_did_type = True
+                    break
+            if not match_did_type:
+                return False
         else:
             if not isinstance(values, list):
                 values = [values, ]
@@ -311,7 +308,7 @@ def transmogrifier(bulk=5, once=False, sleep_time=60):
                                     copies = int(rule_dict['copies'])
                                     activity = rule_dict.get('activity', 'User Subscriptions')
                                     try:
-                                        validate_schema(name='activity', obj=activity)
+                                        validate_schema(name='activity', obj=activity, vo=account.vo)
                                     except InputValidationError as error:
                                         logging.error(prepend_str + 'Error validating the activity %s' % (str(error)))
                                         activity = 'User Subscriptions'
@@ -436,7 +433,7 @@ def transmogrifier(bulk=5, once=False, sleep_time=60):
                     elif did['did_type'] == str(DIDType.CONTAINER):
                         monitor.record_counter(counters='transmogrifier.did.container.processed', delta=1)
                     monitor.record_counter(counters='transmogrifier.did.processed', delta=1)
-                    identifiers.append({'scope': did['scope'], 'name': did['name'], 'did_type': DIDType.from_sym(did['did_type'])})
+                    identifiers.append({'scope': did['scope'], 'name': did['name'], 'did_type': did['did_type']})
 
             time1 = time.time()
 
@@ -471,6 +468,10 @@ def run(threads=1, bulk=100, once=False, sleep_time=60):
     """
     Starts up the transmogrifier threads.
     """
+    setup_logging()
+
+    if rucio.db.sqla.util.is_old_db():
+        raise DatabaseException('Database was not updated, daemon won\'t start')
 
     if once:
         logging.info('Will run only one iteration in a single threaded mode')
