@@ -1,4 +1,5 @@
-# Copyright 2015-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2015-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,14 +14,12 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2012-2017
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012-2020
-# - Angelos Molfetas, <angelos.molfetas@cern.ch>, 2012
-# - Ralph Vigne, <ralph.vigne@cern.ch>, 2013
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2020
-# - Martin Barisits, <martin.barisits@cern.ch>, 2013-2020
-# - Wen Guan, <wen.guan@cern.ch>, 2015
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2015-2019
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2017
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2019
+# - Wen Guan <wen.guan@cern.ch>, 2015
+# - Martin Barisits <martin.barisits@cern.ch>, 2015-2020
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2016-2020
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2017-2020
 # - asket <asket.agarwal96@gmail.com>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
@@ -29,15 +28,13 @@
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - James Perry <j.perry@epcc.ed.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
 
 import datetime
 import sys
 import uuid
 
 from six import iteritems
-from sqlalchemy import BigInteger, Boolean, Column, DateTime, Float, Integer, SmallInteger, String as _String, Text, event, UniqueConstraint
+from sqlalchemy import BigInteger, Boolean, Column, DateTime, Enum, Float, Integer, SmallInteger, String as _String, Text, event, UniqueConstraint
 from sqlalchemy.engine import Engine
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.ext.declarative import declared_attr
@@ -85,19 +82,20 @@ def compile_binary_oracle(type_, compiler, **kw):
     return "NUMBER(1)"
 
 
-# PostgreSQL expects foreign keys to have the same type.
-# Unfortunately, SQLAlchemy propagates the name into the type for the PostgreSQL driver,
-# For now, we need to force rename that one case where this happens.
 @event.listens_for(Table, "before_create")
-def _psql_rename_type(target, connection, **kw):
-    if connection.dialect.name == 'postgresql' and target.name == 'account_map':
-        target.columns.identity_type.type.impl.name = 'IDENTITIES_TYPE_CHK'
-    elif connection.dialect.name == 'mysql' and target.name == 'quarantined_replicas_history':
+def _mysql_rename_type(target, connection, **kw):
+    if connection.dialect.name == 'mysql' and target.name == 'quarantined_replicas_history':
         target.columns.path.type = String(255)
     elif connection.dialect.name == 'mysql' and target.name == 'quarantined_replicas':
         target.columns.path.type = String(255)
 
 
+@event.listens_for(Table, "before_create")
+def _psql_rename_type(target, connection, **kw):
+    if connection.dialect.name == 'postgresql' and target.name == 'account_map':
+        target.columns.identity_type.type.name = 'IDENTITIES_TYPE_CHK'
+
+
 @event.listens_for(Table, "before_create")
 def _oracle_json_constraint(target, connection, **kw):
     if connection.dialect.name == 'oracle' and target.name == 'did_meta':
@@ -175,6 +173,15 @@ def _ck_constraint_name(const, table):
     if const.name is None:
         const.name = table.name.upper() + '_' + str(uuid.uuid4())[:6] + '_CHK'
 
+    if const.name == 'REQUESTS_TYPE_CHK' and table.name.upper() == 'REQUESTS_HISTORY':
+        const.name = "REQUESTS_HISTORY_TYPE_CHK"
+    elif const.name == 'REQUESTS_DIDTYPE_CHK' and table.name.upper() == 'REQUESTS_HISTORY':
+        const.name = "REQUESTS_HISTORY_DIDTYPE_CHK"
+    elif const.name == 'REQUESTS_DIDTYPE_CHK' and table.name.upper() == 'REQUESTS_HISTORY':
+        const.name = "REQUESTS_HISTORY_DIDTYPE_CHK"
+    elif const.name == 'REQUESTS_STATE_CHK' and table.name.upper() == 'REQUESTS_HISTORY':
+        const.name = "REQUESTS_HISTORY_STATE_CHK"
+
 
 @event.listens_for(Table, "after_parent_attach")
 def _add_created_col(table, metadata):
@@ -295,8 +302,11 @@ class Account(BASE, ModelBase):
     """Represents an account"""
     __tablename__ = 'accounts'
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
-    account_type = Column(AccountType.db_type(name='ACCOUNTS_TYPE_CHK'))
-    status = Column(AccountStatus.db_type(default=AccountStatus.ACTIVE, name='ACCOUNTS_STATUS_CHK'))
+    account_type = Column(Enum(AccountType, name='ACCOUNTS_TYPE_CHK',
+                               values_callable=lambda obj: [e.value for e in obj]))
+    status = Column(Enum(AccountStatus, name='ACCOUNTS_STATUS_CHK',
+                         values_callable=lambda obj: [e.value for e in obj]),
+                    default=AccountStatus.ACTIVE, )
     email = Column(String(255))
     suspended_at = Column(DateTime)
     deleted_at = Column(DateTime)
@@ -320,7 +330,8 @@ class Identity(BASE, SoftModelBase):
     """Represents an identity"""
     __tablename__ = 'identities'
     identity = Column(String(2048))
-    identity_type = Column(IdentityType.db_type(name='IDENTITIES_TYPE_CHK'))
+    identity_type = Column(Enum(IdentityType, name='IDENTITIES_TYPE_CHK',
+                                values_callable=lambda obj: [e.value for e in obj]))
     username = Column(String(255))
     password = Column(String(255))
     salt = Column(LargeBinary(255))
@@ -334,7 +345,8 @@ class IdentityAccountAssociation(BASE, ModelBase):
     """Represents a map account-identity"""
     __tablename__ = 'account_map'
     identity = Column(String(2048))
-    identity_type = Column(IdentityType.db_type(name='ACCOUNT_MAP_ID_TYPE_CHK'))
+    identity_type = Column(Enum(IdentityType, name='ACCOUNT_MAP_ID_TYPE_CHK',
+                                values_callable=lambda obj: [e.value for e in obj]))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     is_default = Column(Boolean(name='ACCOUNT_MAP_DEFAULT_CHK'), default=False)
     _table_args = (PrimaryKeyConstraint('identity', 'identity_type', 'account', name='ACCOUNT_MAP_PK'),
@@ -350,7 +362,9 @@ class Scope(BASE, ModelBase):
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     is_default = Column(Boolean(name='SCOPES_DEFAULT_CHK'), default=False)
-    status = Column(ScopeStatus.db_type(name='SCOPE_STATUS_CHK', default=ScopeStatus.OPEN))
+    status = Column(Enum(ScopeStatus, name='SCOPE_STATUS_CHK',
+                         values_callable=lambda obj: [e.value for e in obj]),
+                    default=ScopeStatus.OPEN)
     closed_at = Column(DateTime)
     deleted_at = Column(DateTime)
     _table_args = (PrimaryKeyConstraint('scope', name='SCOPES_SCOPE_PK'),
@@ -366,14 +380,16 @@ class DataIdentifier(BASE, ModelBase):
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
-    did_type = Column(DIDType.db_type(name='DIDS_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='DIDS_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     is_open = Column(Boolean(name='DIDS_IS_OPEN_CHK'))
     monotonic = Column(Boolean(name='DIDS_MONOTONIC_CHK'), server_default='0')
     hidden = Column(Boolean(name='DIDS_HIDDEN_CHK'), server_default='0')
     obsolete = Column(Boolean(name='DIDS_OBSOLETE_CHK'), server_default='0')
     complete = Column(Boolean(name='DIDS_COMPLETE_CHK'), server_default=None)
     is_new = Column(Boolean(name='DIDS_IS_NEW_CHK'), server_default='1')
-    availability = Column(DIDAvailability.db_type(name='DIDS_AVAILABILITY_CHK'),
+    availability = Column(Enum(DIDAvailability, name='DIDS_AVAILABILITY_CHK',
+                               values_callable=lambda obj: [e.value for e in obj]),
                           default=DIDAvailability.AVAILABLE)
     suppressed = Column(Boolean(name='FILES_SUPP_CHK'), server_default='0')
     bytes = Column(BigInteger)
@@ -413,7 +429,6 @@ class DataIdentifier(BASE, ModelBase):
                    CheckConstraint('SUPPRESSED IS NOT NULL', name='DIDS_SUPP_NN'),
                    CheckConstraint('ACCOUNT IS NOT NULL', name='DIDS_ACCOUNT_NN'),
                    CheckConstraint('PURGE_REPLICAS IS NOT NULL', name='DIDS_PURGE_REPLICAS_NN'),
-                   # UniqueConstraint('guid', name='DIDS_GUID_UQ'),
                    Index('DIDS_IS_NEW_IDX', 'is_new'),
                    Index('DIDS_EXPIRED_AT_IDX', 'expired_at'))
 
@@ -433,14 +448,16 @@ class DeletedDataIdentifier(BASE, ModelBase):
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
-    did_type = Column(DIDType.db_type(name='DEL_DIDS_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='DEL_DIDS_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     is_open = Column(Boolean(name='DEL_DIDS_IS_OPEN_CHK'))
     monotonic = Column(Boolean(name='DEL_DIDS_MONO_CHK'), server_default='0')
     hidden = Column(Boolean(name='DEL_DIDS_HIDDEN_CHK'), server_default='0')
     obsolete = Column(Boolean(name='DEL_DIDS_OBSOLETE_CHK'), server_default='0')
     complete = Column(Boolean(name='DEL_DIDS_COMPLETE_CHK'))
     is_new = Column(Boolean(name='DEL_DIDS_IS_NEW_CHK'), server_default='1')
-    availability = Column(DIDAvailability.db_type(name='DEL_DIDS_AVAIL_CHK'),
+    availability = Column(Enum(DIDAvailability, name='DEL_DIDS_AVAIL_CHK',
+                               values_callable=lambda obj: [e.value for e in obj]),
                           default=DIDAvailability.AVAILABLE)
     suppressed = Column(Boolean(name='DEL_FILES_SUPP_CHK'), server_default='0')
     bytes = Column(BigInteger)
@@ -480,7 +497,8 @@ class UpdatedDID(BASE, ModelBase):
     id = Column(GUID(), default=utils.generate_uuid)
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    rule_evaluation_action = Column(DIDReEvaluation.db_type(name='UPDATED_DIDS_RULE_EVAL_ACT_CHK'))
+    rule_evaluation_action = Column(Enum(DIDReEvaluation, name='UPDATED_DIDS_RULE_EVAL_ACT_CHK',
+                                         values_callable=lambda obj: [e.value for e in obj]))
     _table_args = (PrimaryKeyConstraint('id', name='UPDATED_DIDS_PK'),
                    CheckConstraint('SCOPE IS NOT NULL', name='UPDATED_DIDS_SCOPE_NN'),
                    CheckConstraint('NAME IS NOT NULL', name='UPDATED_DIDS_NAME_NN'),
@@ -494,7 +512,9 @@ class BadReplicas(BASE, ModelBase):
     name = Column(String(get_schema_value('NAME_LENGTH')))
     rse_id = Column(GUID())
     reason = Column(String(255))
-    state = Column(BadFilesStatus.db_type(name='BAD_REPLICAS_STATE_CHK'), default=BadFilesStatus.SUSPICIOUS)
+    state = Column(Enum(BadFilesStatus, name='BAD_REPLICAS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=BadFilesStatus.SUSPICIOUS)
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     bytes = Column(BigInteger)
     expires_at = Column(DateTime)
@@ -504,14 +524,17 @@ class BadReplicas(BASE, ModelBase):
                    CheckConstraint('RSE_ID IS NOT NULL', name='BAD_REPLICAS_RSE_ID_NN'),
                    ForeignKeyConstraint(['account'], ['accounts.account'], name='BAD_REPLICAS_ACCOUNT_FK'),
                    Index('BAD_REPLICAS_STATE_IDX', 'rse_id', 'state'),
-                   Index('BAD_REPLICAS_EXPIRES_AT_IDX', 'expires_at'))
+                   Index('BAD_REPLICAS_EXPIRES_AT_IDX', 'expires_at'),
+                   Index('BAD_REPLICAS_ACCOUNT_IDX', 'account'))
 
 
 class BadPFNs(BASE, ModelBase):
     """Represents bad, suspicious or temporary unavailable PFNs which have to be processed and added to BadReplicas Table"""
     __tablename__ = 'bad_pfns'
     path = Column(String(2048))  # PREFIX + PFN
-    state = Column(BadPFNStatus.db_type(name='BAD_PFNS_STATE_CHK'), default=BadPFNStatus.SUSPICIOUS)
+    state = Column(Enum(BadPFNStatus, name='BAD_PFNS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=BadPFNStatus.SUSPICIOUS)
     reason = Column(String(255))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     expires_at = Column(DateTime)
@@ -539,7 +562,8 @@ class DIDKey(BASE, ModelBase):
     __tablename__ = 'did_keys'
     key = Column(String(255))
     is_enum = Column(Boolean(name='DID_KEYS_IS_ENUM_CHK'), server_default='0')
-    key_type = Column(KeyType.db_type(name='DID_KEYS_TYPE_CHK'))
+    key_type = Column(Enum(KeyType, name='DID_KEYS_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     value_type = Column(String(255))
     value_regexp = Column(String(255))
     _table_args = (PrimaryKeyConstraint('key', name='DID_KEYS_PK'),
@@ -563,8 +587,10 @@ class DataIdentifierAssociation(BASE, ModelBase):
     name = Column(String(get_schema_value('NAME_LENGTH')))    # dataset name
     child_scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))  # Provenance scope
     child_name = Column(String(get_schema_value('NAME_LENGTH')))    # Provenance name
-    did_type = Column(DIDType.db_type(name='CONTENTS_DID_TYPE_CHK'))
-    child_type = Column(DIDType.db_type(name='CONTENTS_CHILD_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='CONTENTS_DID_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
+    child_type = Column(Enum(DIDType, name='CONTENTS_CHILD_TYPE_CHK',
+                             values_callable=lambda obj: [e.value for e in obj]))
     bytes = Column(BigInteger)
     adler32 = Column(String(8))
     md5 = Column(String(32))
@@ -576,7 +602,8 @@ class DataIdentifierAssociation(BASE, ModelBase):
                    ForeignKeyConstraint(['child_scope', 'child_name'], ['dids.scope', 'dids.name'], ondelete="CASCADE", name='CONTENTS_CHILD_ID_FK'),
                    CheckConstraint('DID_TYPE IS NOT NULL', name='CONTENTS_DID_TYPE_NN'),
                    CheckConstraint('CHILD_TYPE IS NOT NULL', name='CONTENTS_CHILD_TYPE_NN'),
-                   Index('CONTENTS_CHILD_SCOPE_NAME_IDX', 'child_scope', 'child_name', 'scope', 'name'))
+                   Index('CONTENTS_CHILD_SCOPE_NAME_IDX', 'child_scope', 'child_name', 'scope', 'name'),
+                   Index('CONTENTS_RULE_EVAL_FB_IDX', 'rule_evaluation'))  # Under Oracle this is a FB index
 
 
 class ConstituentAssociation(BASE, ModelBase):
@@ -618,8 +645,6 @@ class ConstituentAssociationHistory(BASE, ModelBase):
         'primary_key': [scope, name, child_scope, child_name]  # Fake primary key for SQLA
     }
     _table_args = (Index('ARCH_CONT_HIST_IDX', 'scope', 'name'), )
-    # _table_args = (PrimaryKeyConstraint('scope', 'name', 'child_scope', 'child_name',
-    #                                     name='ARCH_CONTENTS_HISOTRY_PK'), )
 
 
 class DataIdentifierAssociationHistory(BASE, ModelBase):
@@ -629,8 +654,10 @@ class DataIdentifierAssociationHistory(BASE, ModelBase):
     name = Column(String(get_schema_value('NAME_LENGTH')))  # dataset name
     child_scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))          # Provenance scope
     child_name = Column(String(get_schema_value('NAME_LENGTH')))  # Provenance name
-    did_type = Column(DIDType.db_type(name='CONTENTS_HIST_DID_TYPE_CHK'))
-    child_type = Column(DIDType.db_type(name='CONTENTS_HIST_CHILD_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='CONTENTS_HIST_DID_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
+    child_type = Column(Enum(DIDType, name='CONTENTS_HIST_CHILD_TYPE_CHK',
+                             values_callable=lambda obj: [e.value for e in obj]))
     bytes = Column(BigInteger)
     adler32 = Column(String(8))
     md5 = Column(String(32))
@@ -642,7 +669,6 @@ class DataIdentifierAssociationHistory(BASE, ModelBase):
     __mapper_args__ = {
         'primary_key': [scope, name, child_scope, child_name]  # Fake primary key for SQLA
     }
-    # _table_args = (PrimaryKeyConstraint('scope', 'name', 'child_scope', 'child_name', name='CONTENTS_HIST_PK'),
     _table_args = (CheckConstraint('DID_TYPE IS NOT NULL', name='CONTENTS_HIST_DID_TYPE_NN'),
                    CheckConstraint('CHILD_TYPE IS NOT NULL', name='CONTENTS_HIST_CHILD_TYPE_NN'),
                    Index('CONTENTS_HISTORY_IDX', 'scope', 'name'))
@@ -654,7 +680,9 @@ class RSE(BASE, SoftModelBase):
     id = Column(GUID(), default=utils.generate_uuid)
     rse = Column(String(255))
     vo = Column(String(3), nullable=False, server_default='def')
-    rse_type = Column(RSEType.db_type(name='RSES_TYPE_CHK'), default=RSEType.DISK)
+    rse_type = Column(Enum(RSEType, name='RSES_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]),
+                      default=RSEType.DISK)
     deterministic = Column(Boolean(name='RSE_DETERMINISTIC_CHK'), default=True)
     volatile = Column(Boolean(name='RSE_VOLATILE_CHK'), default=False)
     staging_area = Column(Boolean(name='RSE_STAGING_AREA_CHK'), default=False)
@@ -818,7 +846,9 @@ class RSEFileAssociation(BASE, ModelBase):
     md5 = Column(String(32))
     adler32 = Column(String(8))
     path = Column(String(1024))
-    state = Column(ReplicaState.db_type(name='REPLICAS_STATE_CHK'), default=ReplicaState.UNAVAILABLE)
+    state = Column(Enum(ReplicaState, name='REPLICAS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=ReplicaState.UNAVAILABLE)
     lock_cnt = Column(Integer, server_default='0')
     accessed_at = Column(DateTime)
     tombstone = Column(DateTime)
@@ -830,7 +860,8 @@ class RSEFileAssociation(BASE, ModelBase):
                    CheckConstraint('bytes IS NOT NULL', name='REPLICAS_SIZE_NN'),
                    CheckConstraint('lock_cnt IS NOT NULL', name='REPLICAS_LOCK_CNT_NN'),
                    Index('REPLICAS_TOMBSTONE_IDX', 'tombstone'),
-                   Index('REPLICAS_PATH_IDX', 'path', mysql_length=get_schema_value('NAME_LENGTH')))
+                   Index('REPLICAS_PATH_IDX', 'path', mysql_length=get_schema_value('NAME_LENGTH')),
+                   Index('REPLICAS_STATE_IDX', 'state'))  # Under Oracle this is a FB Index
 
 
 class CollectionReplica(BASE, ModelBase):
@@ -838,13 +869,16 @@ class CollectionReplica(BASE, ModelBase):
     __tablename__ = 'collection_replicas'
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    did_type = Column(DIDType.db_type(name='COLLECTION_REPLICAS_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='COLLECTION_REPLICAS_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     rse_id = Column(GUID())
     bytes = Column(BigInteger)
     length = Column(BigInteger)
     available_bytes = Column(BigInteger)
     available_replicas_cnt = Column(BigInteger)
-    state = Column(ReplicaState.db_type(name='COLLECTION_REPLICAS_STATE_CHK'), default=ReplicaState.UNAVAILABLE)
+    state = Column(Enum(ReplicaState, name='COLLECTION_REPLICAS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=ReplicaState.UNAVAILABLE)
     accessed_at = Column(DateTime)
     _table_args = (PrimaryKeyConstraint('scope', 'name', 'rse_id', name='COLLECTION_REPLICAS_PK'),
                    ForeignKeyConstraint(['scope', 'name'], ['dids.scope', 'dids.name'], name='COLLECTION_REPLICAS_LFN_FK'),
@@ -860,7 +894,8 @@ class UpdatedCollectionReplica(BASE, ModelBase):
     id = Column(GUID(), default=utils.generate_uuid)
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    did_type = Column(DIDType.db_type(name='UPDATED_COL_REP_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='UPDATED_COL_REP_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     rse_id = Column(GUID())
     _table_args = (PrimaryKeyConstraint('id', name='UPDATED_COL_REP_PK'),
                    CheckConstraint('SCOPE IS NOT NULL', name='UPDATED_COL_REP_SCOPE_NN'),
@@ -878,7 +913,6 @@ class RSEFileAssociationHistory(BASE, ModelBase):
     _table_args = (PrimaryKeyConstraint('rse_id', 'scope', 'name', name='REPLICAS_HIST_PK'),
                    ForeignKeyConstraint(['rse_id'], ['rses.id'], name='REPLICAS_HIST_RSE_ID_FK'),
                    CheckConstraint('bytes IS NOT NULL', name='REPLICAS_HIST_SIZE_NN'))
-#  ForeignKeyConstraint(['scope', 'name'], ['dids.scope', 'dids.name'], name='REPLICAS_HIST_LFN_FK'),
 
 
 class ReplicationRule(BASE, ModelBase):
@@ -889,8 +923,11 @@ class ReplicationRule(BASE, ModelBase):
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    did_type = Column(DIDType.db_type(name='RULES_DID_TYPE_CHK'))
-    state = Column(RuleState.db_type(name='RULES_STATE_CHK'), default=RuleState.REPLICATING)
+    did_type = Column(Enum(DIDType, name='RULES_DID_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
+    state = Column(Enum(RuleState, name='RULES_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=RuleState.REPLICATING)
     error = Column(String(255))
     rse_expression = Column(String(3000))
     copies = Column(SmallInteger, server_default='1')
@@ -902,8 +939,12 @@ class ReplicationRule(BASE, ModelBase):
     locks_stuck_cnt = Column(BigInteger, server_default='0')
     source_replica_expression = Column(String(255))
     activity = Column(String(50), default='default')
-    grouping = Column(RuleGrouping.db_type(name='RULES_GROUPING_CHK'), default=RuleGrouping.ALL)
-    notification = Column(RuleNotification.db_type(name='RULES_NOTIFICATION_CHK'), default=RuleNotification.NO)
+    grouping = Column(Enum(RuleGrouping, name='RULES_GROUPING_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]),
+                      default=RuleGrouping.ALL)
+    notification = Column(Enum(RuleNotification, name='RULES_NOTIFICATION_CHK',
+                               values_callable=lambda obj: [e.value for e in obj]),
+                          default=RuleNotification.NO)
     stuck_at = Column(DateTime)
     purge_replicas = Column(Boolean(name='RULES_PURGE_REPLICAS_CHK'), default=False)
     ignore_availability = Column(Boolean(name='RULES_IGNORE_AVAILABILITY_CHK'), default=False)
@@ -934,7 +975,7 @@ class ReplicationRule(BASE, ModelBase):
                          unique=True, mysql_length={'rse_expression': 767}),
                    Index('RULES_SCOPE_NAME_IDX', 'scope', 'name'),
                    Index('RULES_EXPIRES_AT_IDX', 'expires_at'),
-                   Index('RULES_STUCKSTATE_IDX', 'state'),  # This Index is only needed for the STUCK state
+                   Index('RULES_STUCKSTATE_IDX', 'state'),  # This Index is only needed for the STUCK state, there also is FB: RULES_INJECTSTATE_IDX, RULES_APPROVALSTATE_IDX
                    Index('RULES_CHILD_RULE_ID_IDX', 'child_rule_id'))
 
 
@@ -946,8 +987,10 @@ class ReplicationRuleHistoryRecent(BASE, ModelBase):
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    did_type = Column(DIDType.db_type(name='RULES_HIST_RECENT_DIDTYPE_CHK'))
-    state = Column(RuleState.db_type(name='RULES_HIST_RECENT_STATE_CHK'))
+    did_type = Column(Enum(DIDType, name='RULES_HIST_RECENT_DIDTYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
+    state = Column(Enum(RuleState, name='RULES_HIST_RECENT_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]))
     error = Column(String(255))
     rse_expression = Column(String(3000))
     copies = Column(SmallInteger)
@@ -959,8 +1002,10 @@ class ReplicationRuleHistoryRecent(BASE, ModelBase):
     locks_stuck_cnt = Column(BigInteger)
     source_replica_expression = Column(String(255))
     activity = Column(String(50))
-    grouping = Column(RuleGrouping.db_type(name='RULES_HIST_RECENT_GROUPING_CHK'))
-    notification = Column(RuleNotification.db_type(name='RULES_HIST_RECENT_NOTIFY_CHK'))
+    grouping = Column(Enum(RuleGrouping, name='RULES_HIST_RECENT_GROUPING_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
+    notification = Column(Enum(RuleNotification, name='RULES_HIST_RECENT_NOTIFY_CHK',
+                               values_callable=lambda obj: [e.value for e in obj]))
     stuck_at = Column(DateTime)
     purge_replicas = Column(Boolean())
     ignore_availability = Column(Boolean())
@@ -986,8 +1031,10 @@ class ReplicationRuleHistory(BASE, ModelBase):
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    did_type = Column(DIDType.db_type(name='RULES_HISTORY_DIDTYPE_CHK'))
-    state = Column(RuleState.db_type(name='RULES_HISTORY_STATE_CHK'))
+    did_type = Column(Enum(DIDType, name='RULES_HISTORY_DIDTYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
+    state = Column(Enum(RuleState, name='RULES_HISTORY_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]))
     error = Column(String(255))
     rse_expression = Column(String(3000))
     copies = Column(SmallInteger)
@@ -999,8 +1046,10 @@ class ReplicationRuleHistory(BASE, ModelBase):
     locks_stuck_cnt = Column(BigInteger)
     source_replica_expression = Column(String(255))
     activity = Column(String(50))
-    grouping = Column(RuleGrouping.db_type(name='RULES_HISTORY_GROUPING_CHK'))
-    notification = Column(RuleNotification.db_type(name='RULES_HISTORY_NOTIFY_CHK'))
+    grouping = Column(Enum(RuleGrouping, name='RULES_HISTORY_GROUPING_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
+    notification = Column(Enum(RuleNotification, name='RULES_HISTORY_NOTIFY_CHK',
+                               values_callable=lambda obj: [e.value for e in obj]))
     stuck_at = Column(DateTime)
     priority = Column(Integer)
     purge_replicas = Column(Boolean())
@@ -1026,10 +1075,11 @@ class ReplicaLock(BASE, ModelBase):
     rse_id = Column(GUID())
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     bytes = Column(BigInteger)
-    state = Column(LockState.db_type(name='LOCKS_STATE_CHK'), default=LockState.REPLICATING)
+    state = Column(Enum(LockState, name='LOCKS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=LockState.REPLICATING)
     repair_cnt = Column(BigInteger)
     _table_args = (PrimaryKeyConstraint('scope', 'name', 'rule_id', 'rse_id', name='LOCKS_PK'),
-                   # ForeignKeyConstraint(['rse_id', 'scope', 'name'], ['replicas.rse_id', 'replicas.scope', 'replicas.name'], name='LOCKS_REPLICAS_FK'),
                    ForeignKeyConstraint(['rule_id'], ['rules.id'], name='LOCKS_RULE_ID_FK'),
                    ForeignKeyConstraint(['account'], ['accounts.account'], name='LOCKS_ACCOUNT_FK'),
                    CheckConstraint('STATE IS NOT NULL', name='LOCKS_STATE_NN'),
@@ -1045,7 +1095,9 @@ class DatasetLock(BASE, ModelBase):
     rule_id = Column(GUID())
     rse_id = Column(GUID())
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
-    state = Column(LockState.db_type(name='DATASET_LOCKS_STATE_CHK'), default=LockState.REPLICATING)
+    state = Column(Enum(LockState, name='DATASET_LOCKS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=LockState.REPLICATING)
     length = Column(BigInteger)
     bytes = Column(BigInteger)
     accessed_at = Column(DateTime)
@@ -1077,14 +1129,20 @@ class Request(BASE, ModelBase, Versioned):
     """Represents a request for a single file with a third party service"""
     __tablename__ = 'requests'
     id = Column(GUID(), default=utils.generate_uuid)
-    request_type = Column(RequestType.db_type(name='REQUESTS_TYPE_CHK'), default=RequestType.TRANSFER)
+    request_type = Column(Enum(RequestType, name='REQUESTS_TYPE_CHK',
+                               values_callable=lambda obj: [e.value for e in obj]),
+                          default=RequestType.TRANSFER)
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    did_type = Column(DIDType.db_type(name='REQUESTS_DIDTYPE_CHK'), default=DIDType.FILE)
+    did_type = Column(Enum(DIDType, name='REQUESTS_DIDTYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]),
+                      default=DIDType.FILE)
     dest_rse_id = Column(GUID())
     source_rse_id = Column(GUID())
     attributes = Column(String(4000))
-    state = Column(RequestState.db_type(name='REQUESTS_STATE_CHK'), default=RequestState.QUEUED)
+    state = Column(Enum(RequestState, name='REQUESTS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=RequestState.QUEUED)
     external_id = Column(String(64))
     external_host = Column(String(256))
     retry_count = Column(Integer(), server_default='0')
@@ -1117,7 +1175,8 @@ class Request(BASE, ModelBase, Versioned):
                    Index('REQUESTS_TYP_STA_UPD_IDX_OLD', 'request_type', 'state', 'updated_at'),
                    Index('REQUESTS_TYP_STA_UPD_IDX', 'request_type', 'state', 'activity'),
                    Index('REQUESTS_RULEID_IDX', 'rule_id'),
-                   Index('REQUESTS_EXTERNALID_UQ', 'external_id'))
+                   Index('REQUESTS_EXTERNALID_UQ', 'external_id'),
+                   Index('REQUESTS_DEST_RSE_ID_IDX', 'dest_rse_id'))
 
 
 class Source(BASE, ModelBase, Versioned):
@@ -1176,7 +1235,9 @@ class Subscription(BASE, ModelBase, Versioned):
     filter = Column(String(2048))
     replication_rules = Column(String(1024))
     policyid = Column(SmallInteger, server_default='0')
-    state = Column(SubscriptionState.db_type(name='SUBSCRIPTIONS_STATE_CHK', default=SubscriptionState.ACTIVE))
+    state = Column(Enum(SubscriptionState, name='SUBSCRIPTIONS_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]),
+                   default=SubscriptionState.ACTIVE)
     last_processed = Column(DateTime, default=datetime.datetime.utcnow())
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     lifetime = Column(DateTime)
@@ -1187,7 +1248,8 @@ class Subscription(BASE, ModelBase, Versioned):
                    UniqueConstraint('name', 'account', name='SUBSCRIPTIONS_NAME_ACCOUNT_UQ'),
                    ForeignKeyConstraint(['account'], ['accounts.account'], name='SUBSCRIPTIONS_ACCOUNT_FK'),
                    CheckConstraint('RETROACTIVE IS NOT NULL', name='SUBSCRIPTIONS_RETROACTIVE_NN'),
-                   CheckConstraint('ACCOUNT IS NOT NULL', name='SUBSCRIPTIONS_ACCOUNT_NN'))
+                   CheckConstraint('ACCOUNT IS NOT NULL', name='SUBSCRIPTIONS_ACCOUNT_NN'),
+                   Index('SUBSCRIPTIONS_STATE_IDX', 'state'))  # Under Oracle this is a FB index
 
 
 class Token(BASE, ModelBase):
@@ -1288,7 +1350,8 @@ class NamingConvention(BASE, ModelBase):
     __tablename__ = 'naming_conventions'
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     regexp = Column(String(255))
-    convention_type = Column(KeyType.db_type(name='CVT_TYPE_CHK'))
+    convention_type = Column(Enum(KeyType, name='CVT_TYPE_CHK',
+                                  values_callable=lambda obj: [e.value for e in obj]))
     _table_args = (PrimaryKeyConstraint('scope', name='NAMING_CONVENTIONS_PK'),
                    ForeignKeyConstraint(['scope'], ['scopes.scope'], name='NAMING_CONVENTIONS_SCOPE_FK'))
 
@@ -1321,11 +1384,13 @@ class LifetimeExceptions(BASE, ModelBase):
     id = Column(GUID(), default=utils.generate_uuid)
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
-    did_type = Column(DIDType.db_type(name='LIFETIME_EXCEPT_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='LIFETIME_EXCEPT_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     pattern = Column(String(255))
     comments = Column(String(4000))
-    state = Column(LifetimeExceptionsState.db_type(name='LIFETIME_EXCEPT_STATE_CHK'))
+    state = Column(Enum(LifetimeExceptionsState, name='LIFETIME_EXCEPT_STATE_CHK',
+                        values_callable=lambda obj: [e.value for e in obj]))
     expires_at = Column(DateTime)
     _table_args = (PrimaryKeyConstraint('id', 'scope', 'name', 'did_type', 'account', name='LIFETIME_EXCEPT_PK'),
                    CheckConstraint('SCOPE IS NOT NULL', name='LIFETIME_EXCEPT_SCOPE_NN'),
@@ -1349,7 +1414,8 @@ class DidsFollowed(BASE, ModelBase):
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
-    did_type = Column(DIDType.db_type(name='DIDS_FOLLOWED_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='DIDS_FOLLOWED_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     _table_args = (PrimaryKeyConstraint('scope', 'name', 'account', name='DIDS_FOLLOWED_PK'),
                    CheckConstraint('SCOPE IS NOT NULL', name='DIDS_FOLLOWED_SCOPE_NN'),
                    CheckConstraint('NAME IS NOT NULL', name='DIDS_FOLLOWED_NAME_NN'),
@@ -1365,7 +1431,8 @@ class FollowEvents(BASE, ModelBase):
     scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     name = Column(String(get_schema_value('NAME_LENGTH')))
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
-    did_type = Column(DIDType.db_type(name='DIDS_FOLLOWED_EVENTS_TYPE_CHK'))
+    did_type = Column(Enum(DIDType, name='DIDS_FOLLOWED_EVENTS_TYPE_CHK',
+                           values_callable=lambda obj: [e.value for e in obj]))
     event_type = Column(String(1024))
     payload = Column(Text)
     _table_args = (PrimaryKeyConstraint('scope', 'name', 'account', name='DIDS_FOLLOWED_EVENTS_PK'),
