@@ -15,21 +15,18 @@
 #
 # Authors:
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2018-2021
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
-from __future__ import print_function
-
 import itertools
+import logging
 import re
 from functools import wraps
 from time import time
-from traceback import format_exc
 
-import six
 from flask import request, Response
 
 from rucio.api.authentication import validate_auth_token
@@ -43,14 +40,14 @@ def request_auth_env():
     if request.environ.get('REQUEST_METHOD') == 'OPTIONS':
         return '', 200
 
-    auth_token = request_header_ensure_string('X-Rucio-Auth-Token')
+    auth_token = request.headers.get('X-Rucio-Auth-Token', default=None)
 
     try:
         auth = validate_auth_token(auth_token)
     except RucioException as error:
         return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
     except Exception as error:
-        print(format_exc())
+        logging.exception("Internal Error")
         return str(error), 500
 
     if auth is None:
@@ -96,17 +93,18 @@ def check_accept_header_wrapper_flask(supported_content_types):
     return wrapper
 
 
-def parse_scope_name(scope_name):
+def parse_scope_name(scope_name, vo):
     """
     Parses the given scope_name according to the schema's
     SCOPE_NAME_REGEXP and returns a (scope, name) tuple.
 
     :param scope_name: the scope_name string to be parsed.
+    :param vo: the vo currently in use.
     :raises ValueError: when scope_name could not be parsed.
     :returns: a (scope, name) tuple.
     """
     # why again does that regex start with a slash?
-    scope_name = re.match(get_schema_value('SCOPE_NAME_REGEXP'), '/' + scope_name)
+    scope_name = re.match(get_schema_value('SCOPE_NAME_REGEXP', vo), '/' + scope_name)
     if scope_name is None:
         raise ValueError('cannot parse scope and name')
     return scope_name.group(1, 2)
@@ -132,24 +130,3 @@ def try_stream(generator, content_type=None):
         return Response(itertools.chain((peek,), it), content_type=content_type)
     except StopIteration:
         return Response('', content_type=content_type)
-
-
-def request_header_ensure_string(key, default=None):
-    """
-    Supplement for request.headers.get(...), which returns
-    unicode strings for Python 2.
-
-    :param key: the header name (case-insensitive).
-    :param default: the value to return, if the header is absent.
-        Returns None by default.
-    :raises TypeError: when the header value was not of binary type.
-    :returns: default, if the key is not present or a str type
-        corresponding to the header's value.
-    """
-    hdrval = request.headers.get(key, default=default, as_bytes=True)
-    if hdrval is None or hdrval == default:
-        return hdrval
-    elif isinstance(hdrval, six.binary_type):
-        return six.ensure_str(hdrval)
-    else:
-        raise TypeError("Unexpected header value type: " + str(type(hdrval)))
