@@ -17,17 +17,16 @@
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2017
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2018-2021
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2019
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2019-2020
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-
-from __future__ import print_function
+# - Martin Barisits <martin.barisits@cern.ch>, 2020
 
 import itertools
-from traceback import format_exc
+import logging
 
 from flask import Flask, Blueprint, request, redirect
 from flask.views import MethodView
@@ -35,8 +34,8 @@ from werkzeug.datastructures import Headers
 
 from rucio.api.replica import list_replicas
 from rucio.common.exception import RucioException, DataIdentifierNotFound, ReplicaNotFound
-from rucio.common.replica_sorter import sort_random, sort_geoip, sort_closeness, sort_ranking, sort_dynamic, site_selector
-from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, parse_scope_name, try_stream, request_header_ensure_string
+from rucio.core.replica_sorter import site_selector, sort_replicas
+from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, parse_scope_name, try_stream
 from rucio.web.rest.utils import generate_http_error_flask
 
 try:
@@ -71,17 +70,17 @@ class MetaLinkRedirector(MethodView):
         headers.set('Access-Control-Allow-Credentials', 'true')
 
         try:
-            scope, name = parse_scope_name(scope_name)
+            scope, name = parse_scope_name(scope_name, request.headers.get('X-Rucio-VO', default='def'))
         except ValueError as error:
             return generate_http_error_flask(400, 'ValueError', error.args[0], headers=headers)
         except Exception as error:
-            print(format_exc())
+            logging.exception("Internal Error")
             return str(error), 500, headers
 
-        dids, schemes, select = [{'scope': scope, 'name': name}], ['http', 'https', 'root', 'gsiftp', 'srm', 'davs'], None
+        dids, schemes, sortby = [{'scope': scope, 'name': name}], ['http', 'https', 'root', 'gsiftp', 'srm', 'davs'], None
 
         # set the correct client IP
-        client_ip = request_header_ensure_string('X-Forwarded-For', request.remote_addr)
+        client_ip = request.headers.get('X-Forwarded-For', default=request.remote_addr)
 
         client_location = {'ip': client_ip,
                            'fqdn': None,
@@ -93,9 +92,9 @@ class MetaLinkRedirector(MethodView):
             if 'schemes' in params:
                 schemes = params['schemes']
             if 'select' in params:
-                select = params['select'][0]
+                sortby = params['select'][0]
             if 'sort' in params:
-                select = params['sort'][0]
+                sortby = params['sort'][0]
 
             if 'ip' in params:
                 client_location['ip'] = params['ip'][0]
@@ -105,7 +104,7 @@ class MetaLinkRedirector(MethodView):
                 client_location['site'] = params['site'][0]
 
         # get vo if given
-        vo = request_header_ensure_string('X-Rucio-VO', 'def')
+        vo = request.headers.get('X-Rucio-VO', default='def')
 
         try:
             replicas_iter = list_replicas(dids=dids, schemes=schemes, client_location=client_location, vo=vo)
@@ -141,17 +140,7 @@ class MetaLinkRedirector(MethodView):
                     yield '  <glfn name="/atlas/rucio/%s:%s">' % (rfile['scope'], rfile['name'])
                     yield '</glfn>\n'
 
-                    # sort the actual replicas if necessary
-                    if select == 'geoip':
-                        replicas = sort_geoip(dictreplica, client_location['ip'], ignore_error=True)
-                    elif select == 'closeness':
-                        replicas = sort_closeness(dictreplica, client_location)
-                    elif select == 'dynamic':
-                        replicas = sort_dynamic(dictreplica, client_location)
-                    elif select == 'ranking':
-                        replicas = sort_ranking(dictreplica, client_location)
-                    else:
-                        replicas = sort_random(dictreplica)
+                    replicas = sort_replicas(dictreplica, client_location, selection=sortby)
 
                     # stream URLs
                     idx = 1
@@ -172,7 +161,7 @@ class MetaLinkRedirector(MethodView):
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0], headers=headers)
         except Exception as error:
-            print(format_exc())
+            logging.exception("Internal Error")
             return str(error), 500, headers
 
 
@@ -199,11 +188,11 @@ class HeaderRedirector(MethodView):
         headers.set('Access-Control-Allow-Credentials', 'true')
 
         try:
-            scope, name = parse_scope_name(scope_name)
+            scope, name = parse_scope_name(scope_name, request.headers.get('X-Rucio-VO', default='def'))
         except ValueError as error:
             return generate_http_error_flask(400, 'ValueError', error.args[0], headers=headers)
         except Exception as error:
-            print(format_exc())
+            logging.exception("Internal Error")
             return str(error), 500, headers
 
         try:
@@ -211,7 +200,7 @@ class HeaderRedirector(MethodView):
             # use the default HTTP protocols if no scheme is given
             select, rse, site, schemes = 'random', None, None, ['davs', 'http', 'https']
 
-            client_ip = request_header_ensure_string('X-Forwarded-For', request.remote_addr)
+            client_ip = request.headers.get('X-Forwarded-For', default=request.remote_addr)
 
             client_location = {'ip': client_ip,
                                'fqdn': None,
@@ -249,7 +238,7 @@ class HeaderRedirector(MethodView):
                 schemes = [schemes]  # list_replicas needs a list
 
             # get vo if given
-            vo = request_header_ensure_string('X-Rucio-VO', 'def')
+            vo = request.headers.get('X-Rucio-VO', default='def')
 
             replicas = [r for r in list_replicas(dids=[{'scope': scope, 'name': name, 'type': 'FILE'}],
                                                  schemes=schemes, client_location=client_location, vo=vo)]
@@ -283,17 +272,7 @@ class HeaderRedirector(MethodView):
                             else:
                                 return 'no redirection possible - no valid RSE for HTTP redirection found', 404, headers
                         else:
-                            if select == 'geoip':
-                                rep = sort_geoip(dictreplica, client_location['ip'])
-                            elif select == 'closeness':
-                                rep = sort_closeness(dictreplica, client_location)
-                            elif select == 'dynamic':
-                                rep = sort_dynamic(dictreplica, client_location)
-                            elif select == 'ranking':
-                                rep = sort_ranking(dictreplica, client_location)
-                            else:
-                                rep = sort_random(dictreplica)
-
+                            rep = sort_replicas(dictreplica, client_location, selection=select)
                             selected_url = rep[0]
 
             if selected_url:
@@ -307,7 +286,7 @@ class HeaderRedirector(MethodView):
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0], headers=headers)
         except Exception as error:
-            print(format_exc())
+            logging.exception("Internal Error")
             return str(error), 500, headers
 
 
